<?php
class backendModel extends model
{
	/**
	 * select active, blocked, suspended, suspended, unapproved, not paid listings
	 *
	 * @return array
	 */
	public function getListingsSummary()
	{
		$this->db->select('status');
		$this->db->select('count(if(status=1, 1, null)) AS active_status', false);
		$this->db->select('count(if(status=2, 1, null)) AS blocked_status');
		$this->db->select('count(if(status=3, 1, null)) AS suspended_status');
		$this->db->select('count(if(status=4, 1, null)) AS unapproved_status');
		$this->db->select('count(if(status=5, 1, null)) AS not_paid_status');
		$this->db->from('listings');
		$this->db->group_by('status');
		$query = $this->db->get();
		
		return $query->result_array();
	}
	/**
	 * select  self active, blocked, suspended, suspended, unapproved, not paid listings
	 *
	 * @return array
	 */
	public function getMyListingsSummary()
	{
		$this->db->select('status');
		$this->db->select('count(if(status=1, 1, null)) AS active_status', false);
		$this->db->select('count(if(status=2, 1, null)) AS blocked_status');
		$this->db->select('count(if(status=3, 1, null)) AS suspended_status');
		$this->db->select('count(if(status=4, 1, null)) AS unapproved_status');
		$this->db->select('count(if(status=5, 1, null)) AS not_paid_status');
		$this->db->from('listings');
		$this->db->group_by('status');
		$this->db->where('owner_id', $this->session->userdata('user_id'));
		$query = $this->db->get();
		
		return $query->result_array();
	}
	/**
	 * select active, blocked users
	 *
	 * @return array
	 */
	public function getUsersSummary()
	{
		$this->db->select('status');
		$this->db->select('count(if(status=1, 1, null)) AS unverified_status');
		$this->db->select('count(if(status=2, 1, null)) AS active_status');
		$this->db->select('count(if(status=3, 1, null)) AS blocked_status');
		$this->db->from('users');
		$this->db->group_by('status');
		$query = $this->db->get();
		
		return $query->result_array();
	}
	/**
	 * select users counts by users groups
	 *
	 * @return array
	 */
	public function getUsersGroupsSummary()
	{
		$this->db->select('ug.id');
		$this->db->select('ug.name');
		$this->db->select('count(*) AS users_count_in_group');
		$this->db->join('users_groups AS ug', 'u.group_id=ug.id', 'left');
		$this->db->from('users AS u');
		$this->db->group_by('ug.id');
		$query = $this->db->get();
		
		return $query->result_array();
	}

	public function getClaimsUnapproved()
	{
		$this->db->select('count(*) AS unapproved_claims');
		$this->db->from('listings_claims');
		$this->db->where('approved', 0);
		$this->db->where('to_user_id !=', 0);
		$row = $this->db->get()->row_array();
		
		return $row['unapproved_claims'];
	}
	
	public function languagesFilesUpdate($uversion, $udir)
	{
		$i18n_dir = $udir . 'i18n/';
		if (is_dir($i18n_dir)) {
			$languages = directory_map(LANGPATH);
		    $map = directory_map($i18n_dir);
			foreach ($map as $file) {
				foreach ($languages AS $lang_code=>$folder) {
					if (is_dir(LANGPATH . $lang_code)) {
						$translation_constants = file_get_contents($i18n_dir . $file);
						preg_match_all("/language\['([A-Z0-9-_]*)'\] = \"(.*)\";/", $translation_constants, $matches);
						foreach ($matches[1] AS $key=>$const_index) {
					    	$q = $matches[2][$key];
		
					    	if ($lang_code != 'en') {
					    		$languagePair = "en|" . $lang_code;
					    		$fullUrl = sprintf("http://ajax.googleapis.com/ajax/services/language/translate?v=1.0&q=%s&langpair=%s", rawurlencode($q), rawurlencode($languagePair));
								/*$response = file_get_contents(
									$fullUrl,
									null,
									stream_context_create(
										array(
											'http'=>array(
												'method'=>"GET",
												'header'=>"Referer: http://".$_SERVER['HTTP_HOST']."/\r\n"
											)
										)
									)
								);*/
								
								$curl_handle=curl_init();
								curl_setopt($curl_handle, CURLOPT_URL, $fullUrl);
								curl_setopt($curl_handle, CURLOPT_CONNECTTIMEOUT, 2);
								curl_setopt($curl_handle, CURLOPT_RETURNTRANSFER, 1);
								curl_setopt($curl_handle, CURLOPT_REFERER, "http://".$_SERVER['HTTP_HOST']);
								$response = curl_exec($curl_handle);
								curl_close($curl_handle);
		
								if (preg_match("/{\"translatedText\":\"([^\"]+)\"/i", $response, $response_matches)) {
									$translated_const = preg_replace_callback("/\\\u([0-9a-f]{4})/i", create_function('$matches', 'return html_entity_decode(\'&#x\'.$matches[1].\';\', ENT_NOQUOTES, \'UTF-8\');'), $response_matches[1]);
								} else {
									$translated_const = '';
								}
					    	} else {
					    		$translated_const = $q;
					    	}
		
							$translation_constants = preg_replace("/language\['$const_index'\] = \"(.*)\";/", "language['$const_index'] = \"$translated_const\";", $translation_constants);
					    }
		
						$dest_file = LANGPATH . $lang_code . '/' . $file;
						/*if (!is_writeable($dest_file)) {
							exit("File: '" . $dest_file . "' must be writeable!");
						}*/

						// Update existed lang file or copy
						if (is_file($dest_file)) {
							$dest_constants = file_get_contents($dest_file);
							$dest_constants = trim($dest_constants, '<?php');
							$dest_constants = trim($dest_constants, '?>');
							$dest_constants .= "
	
	/* Update to " . $uversion . " */
	";
							$translation_constants = trim($translation_constants, '<?php');
							$translation_constants = trim($translation_constants, '?>');
							$dest_constants .= $translation_constants;
						    file_put_contents($dest_file, '<?php
	'.$dest_constants.'
	?>');
						} else {
							@copy($i18n_dir . $file, $dest_file);
							@chmod($dest_file, 0777);
						}
					}
				}
			}

			// --------------------------------------------------------------------------------------------
	    	// In order to avoid 'MYSQL server has gone away' problem
	    	// --------------------------------------------------------------------------------------------
	    	$this->db->reconnect();
	    	// --------------------------------------------------------------------------------------------
		}
	}
}
?>