<?php

class category
{
	public $id;
	public $name;
	public $seo_name;
	public $selected_icons_serialized;
	public $meta_title;
	public $meta_description;
	public $type_id;
	public $parent_category_id;
	public $tree_path;
	public $counter = 0;
	public $children = array();
	
	public function __construct()
	{
		$this->id = 'new';
		$this->name = '';
		$this->seo_name = '';
		$this->selected_icons_serialized = '';
		$this->meta_title = '';
		$this->meta_description = '';
		$this->parent_category_id = 0;
	}
	
	public function setCategoryFromArray($array)
	{
		if (isset($array['id']))
			$this->id = $array['id'];
		$this->name = $array['name'];
		$this->seo_name = $array['seo_name'];
		$this->selected_icons_serialized = $array['selected_icons_serialized'];
		$this->meta_title = $array['meta_title'];
		$this->meta_description = $array['meta_description'];
		if (isset($array['type_id']))
			$this->type_id = $array['type_id'];
		if (isset($array['parent_category_id']))
			$this->parent_category_id = $array['parent_category_id'];
		if (isset($array['tree_path']))
			$this->tree_path = $array['tree_path'];
	}
	
	public function getChainAsArray()
	{
		$CI = &get_instance();
    	$CI->load->model('categories', 'categories');

		$chain_ids = explode('-', $this->tree_path);
		array_shift($chain_ids);
		
		$chain = array();
		foreach ($chain_ids AS $category_id) {
			if ($category_id) {
				$category = $CI->categories->getCategoryById($category_id);
				$chain[] = $category;
			}
		}
		return $chain;
	}
	
	public function getChainAsString($glue = ' » ')
	{
		$chain = array();
		$array = $this->getChainAsArray();
		foreach ($array AS $category) {
			$chain[] = $category->name;
		}
		return implode($glue, $chain);
	}
	
	public function getChainAsUrl()
	{
		$chain = array();
		$array = $this->getChainAsArray();
		foreach ($array AS $category) {
			$chain[] = $category->seo_name;
		}

		// If this is local category
		if ($this->type_id) {
			$CI = &get_instance();
			$CI->load->model('types', 'types');
			$type = $CI->types->getTypeById($this->type_id);
			
			$url = 'type/' . $type->seo_name . '/';
		} else {
			$url = '';
		}

		$url .= 'categories/' . implode('/', $chain) . '/';
		return $url;
	}
	
	public function getChainAsLinks($glue = ' » ')
	{
		$chain = array();
		$array = $this->getChainAsArray();
		foreach ($array AS $category) {
			$chain[] = '<a href="' . site_url($category->getUrl()) . '" >' . $category->name . '</a>';
		}
		return implode($glue, $chain);
	}
	
	public function getUrl()
	{
		$CI = &get_instance();
		if ($current_location = $CI->session->userdata('current_location_seo_name')) {
			return 'location/' . $current_location . '/' . $this->getChainAsUrl();
		} else {
			return $this->getChainAsUrl();
		}
	}
	
	public function buildChildren($categories_array, $max_depth = 'max')
	{
		foreach ($categories_array AS $category_row) {
			if ($this->id == $category_row['parent_category_id'] && ($max_depth == 'max' || count(array_filter(explode('-', $category_row['tree_path']))) <= $max_depth)) {
				$category = new category;
				$category->setCategoryFromArray($category_row);
				$category->buildChildren($categories_array, $max_depth);
				$this->children[] = $category;
			}
		}
	}
	
	/**
	 * Counts all listings inside this category and all its children
	 *
	 * @return int
	 */
	public function countListings()
	{
		if (!$this->counter) {
			$CI = &get_instance();
			$CI->db->distinct();
			$CI->db->select('lic.listing_id');
			$CI->db->from('listings_in_categories AS lic');
			$CI->db->join('categories AS c', 'c.id=lic.category_id', 'left');
			$CI->db->join('listings AS l', 'l.id=lic.listing_id', 'left');
			$CI->db->join('users AS u', 'u.id=l.owner_id', 'left');
			if ($current_location = registry::get('current_location')) {
				$CI->db->join('listings_in_locations AS lil', 'lil.listing_id=l.id', 'left');
	    		$CI->db->where('(lil.geocoded_name LIKE "%'.$current_location->geocoded_name.'")', null, false);
			}
			$CI->db->where('(c.tree_path LIKE "%-'.$this->id.'-%" OR c.tree_path LIKE "%-'.$this->id.'")', null, false);
			$CI->db->where('l.status', 1);
			$CI->db->where('u.status', 2);
			$array = $CI->db->get()->result_array();
			foreach ($array AS $row) {
				$this->counter ++;
			}
		}
		return $this->counter;
	}
	
	public function render($template, $is_counter = false, $selected_categories = null, $highlight_element = '', $is_children_label = '')
	{
		$tokens = array(
			'ID',
			'SEONAME',
			'URL',
			'NAME',
			'CHILDREN',
			'HIGHLIGHT',
			'COUNTER',
			'OBRACKET',
			'CBRACKET',
			'AJAXURL',
			'ISCHILDRENLABEL',
		);
		$_template = $template;
		foreach ($tokens AS $token) {
			$function_name = '_get' . ucfirst(strtolower($token));
	    	$_template = str_ireplace('%'.$token.'%', $this->$function_name($template, $is_counter, $selected_categories, $highlight_element, $is_children_label), $_template);
		}

		return $_template;
	}
	
	public function _getObracket()
	{
		return '{';
	}
	public function _getCbracket()
	{
		return '}';
	}
	
	public function _getId()
	{
		return $this->id;
	}
	
	public function _getSeoname()
	{
		return $this->seo_name;
	}
	
	public function _getUrl()
	{
		return site_url($this->getUrl());
	}

	public function _getName()
	{
		return $this->name;
	}
	
	public function _getChildren($template, $is_counter, $selected_categories, $highlight_element)
	{
		$children_output = '';
		if ($this->children) {
			foreach ($this->children AS $child) {
				$children_output .= $child->render($template, $is_counter, $selected_categories, $highlight_element);
			}
		}
		return $children_output;
	}
	
	public function _getHighlight($template, $is_counter, $selected_categories, $highlight_element)
	{
		if (is_array($selected_categories)) {
			if ((!in_array($this->id, $selected_categories)) && (!in_array($this->seo_name, $selected_categories)))
				$highlight_element = '';
		} elseif ($selected_categories != $this->seo_name)
				$highlight_element = '';
		return $highlight_element;
	}
	
	public function _getCounter($template, $is_counter)
	{
		if ($is_counter) {
			return '&nbsp;(' . $this->countListings() . ')';
		} else
			return '';
	}
	
	public function _getIschildrenlabel($template, $is_counter, $selected_categories, $highlight_element, $is_children_label)
	{
		if ($this->children) {
			return $is_children_label;
		} else {
			return '';
		}
	}
	
	public function _getAjaxurl()
	{
		if ($this->type_id)
			return site_url("ajax/categories_request/frontend/" . $this->type_id);
		else
			return site_url("ajax/categories_request/frontend/");
	}
	
	/**
	 * returns category with the same seo name, but of another local type
	 *
	 * @param int/string $type
	 */
	public function getSimilarCategoryOfAnotherType($type)
	{
		$CI = &get_instance();
		$CI->load->model('types', 'types_levels');
		$CI->load->model('categories', 'categories');
		
		if (is_numeric($type)) {
			$type = $CI->types->getTypeById($type);
		} elseif (is_string($type)) {
			$type = $CI->types->getTypeBySeoName($type);
		} else {
			return ;
		}
		
		$CI->categories->setTypeId($type->id);
		return $CI->categories->getCategoryBySeoName($this->seo_name);
	}
}
?>