<?php
include_once(MODULES_PATH . 'i18n/classes/language.class.php');

class languagesModel extends Model 
{
	private $_language_id;

	public function getLanguages()
	{
		$this->db->select();
		$this->db->from('languages');
		$this->db->orderBy('order_num');
		$query = $this->db->get();
		return $query->result_array();
	}
	
	public function setLanguageId($language_id)
	{
		$this->_language_id = $language_id;
	}

	public function setLanguagesOrder($serialized_order)
    {
    	$a = explode("=", $serialized_order);
    	$i = 1;
    	foreach ($a AS $row) {
    		$b = explode("&", $row);
    		foreach ($b AS $id) {
    			$id = trim($id, "_id");
    			if (is_numeric($id)) {
    				$this->db->set('order_num', $i++);
    				$this->db->where('id', $id);
    				$this->db->update('languages');
    			}
    		}
    	}
    }
    
    /**
     * is there language with such name in the DB?
     *
     * @param string $name
     */
    public function is_name($name)
    {
    	$this->db->select();
		$this->db->from('languages');
		$this->db->where('name', $name);
		if (!is_null($this->_language_id)) {
			$this->db->where('id !=', $this->_language_id);
		}
		$query = $this->db->get();

		return $query->num_rows();
    }
    
    /**
     * is there language with such code in the DB?
     *
     * @param string $name
     */
    public function is_code($code)
    {
    	$this->db->select();
		$this->db->from('languages');
		$this->db->where('code', $code);
		if (!is_null($this->_language_id)) {
			$this->db->where('id !=', $this->_language_id);
		}
		$query = $this->db->get();

		return $query->num_rows();
    }
    
    /**
     * is there language with such db code?
     *
     * @param string $name
     */
    public function is_db_code($db_code)
    {
    	$this->db->select();
		$this->db->from('languages');
		$this->db->where('db_code', $db_code);
		if (!is_null($this->_language_id)) {
			$this->db->where('id !=', $this->_language_id);
		}
		$query = $this->db->get();

		return $query->num_rows();
    }
    
    public function saveLanguage($form)
	{
		$this->db->select_max('order_num');
    	$query = $this->db->get('languages');
    	if ($row = $query->row())
    		$order_num = $row->order_num + 1;
    	else 
    		$order_num = 1;

    	$this->db->set('active', 1);
		$this->db->set('name', $form['name']);
		$this->db->set('code', $form['code']);
		$this->db->set('db_code', $form['db_code']);
		$this->db->set('flag', $form['flag']);
		$this->db->set('custom_theme', $form['custom_theme']);
		$this->db->set('decimals_separator', $form['decimals_separator']);
		$this->db->set('thousands_separator', $form['thousands_separator']);
		$this->db->set('date_format', $form['date_format']);
		$this->db->set('time_format', $form['time_format']);
		$this->db->set('order_num', $order_num);
		if ($this->db->insert('languages')) {
			return $this->addLangColumns($form['db_code']);
		}
	}
	
	public function getLanguageFromForm($form)
	{
		$language = new language;
        $language->setLanguageFromArray($form);
        return $language;
	}
	
	public function getNewLanguage()
	{
		$language = new language;
        return $language;
	}
	
	public function getAllFlags()
	{
		$system_settings = registry::get('system_settings');
		$flags = array();
		$path = 'themes/' . $system_settings['design_theme'] . '/images/flags/';

		$this->load->helper('directory');
		$map = directory_map($path);
		foreach ($map as $file) {
			$flags[] = $file;
		}
		sort($flags);
		return $flags;
	}
	
	public function createLanguageDir($code, $manual)
	{
		$this->load->helper('directory');
		$system_settings = registry::get('system_settings');
        $default_language_code = $system_settings['default_language'];

        $default_dir = LANGPATH . $default_language_code;
		$new_dir = LANGPATH . $code;

		@mkdir($new_dir);
		$map = directory_map($default_dir);
		foreach ($map as $file) {
			if (!$manual) {
				$this->autoTranslateLanguageFile($default_language_code, $code, $file);
			} else {
				$this->copyLanguageFile($default_language_code, $code, $file);
			}
		}
		return true;
	}
	
	public function autoTranslateLanguageFile($default_language_code, $language_code, $file_name, $lang_dir = null)
    {
    	// 'en' to 'en' folder
    	if ($default_language_code == $language_code) {
    		if ($lang_dir && is_file($lang_dir . DIRECTORY_SEPARATOR . $file_name)) {
    			if (copy($lang_dir . DIRECTORY_SEPARATOR . $file_name, LANGPATH . $language_code . DIRECTORY_SEPARATOR . $file_name) &&
    				@chmod(LANGPATH . $language_code . DIRECTORY_SEPARATOR . $file_name, 0777))
    				return true;
    			else 
    				return false;
    		}
    	}
    	
    	if ($lang_dir && is_file($lang_dir . DIRECTORY_SEPARATOR . $file_name))
        	$translation_constants = file_get_contents($lang_dir . DIRECTORY_SEPARATOR . $file_name);
    	else
        	$translation_constants = file_get_contents(LANGPATH . $default_language_code . DIRECTORY_SEPARATOR . $file_name);

    	preg_match_all("/language\['([A-Z0-9-_]*)'\] = \"(.*)\";/", $translation_constants, $matches);

    	$languagePair = $default_language_code . "|" . $language_code;
    	$url = "http://ajax.googleapis.com/ajax/services/language/translate?v=1.0&langpair=" . rawurlencode($languagePair);
    	
    	$chars_count = 0;
    	// balance of translation speed
    	$max_chars = 500;
    	$need_to_translate = array();
    	$i = 0;
    	$count = 0;
    	foreach ($matches[1] AS $key=>$const_index) {
    		$count++;
    		if (($chars_count+strlen($matches[2][$key])<$max_chars || $i==0) && count($matches[1])!=$count) {
    			$chars_count += strlen($matches[2][$key]);
    			$need_to_translate[1][$i] = $const_index;
    			$need_to_translate[2][$i] = $matches[2][$key];
    			$i++;
    		} else {
    			$fullUrl = $url;

				$chars_count += strlen($matches[2][$key]);
    			$need_to_translate[1][$i] = $const_index;
    			$need_to_translate[2][$i] = $matches[2][$key];

    			foreach ($need_to_translate[1] AS $key=>$const_index) {
		    		$q = $need_to_translate[2][$key];
		    		$fullUrl .= "&q=".rawurlencode($q);
		    	}

				$curl_handle=curl_init();
				curl_setopt($curl_handle, CURLOPT_URL, $fullUrl);
				curl_setopt($curl_handle, CURLOPT_CONNECTTIMEOUT, 2);
				curl_setopt($curl_handle, CURLOPT_RETURNTRANSFER, 1);
				curl_setopt($curl_handle, CURLOPT_REFERER, "http://".$_SERVER['HTTP_HOST']);
				$response = curl_exec($curl_handle);
				curl_close($curl_handle);

		    	$translated_const_array = array();
		    	$obj = json_decode($response);
		    	$res_array = $obj->responseData;
		    	foreach ($need_to_translate[1] AS $key=>$const_index) {
			    	if (is_array($res_array) && $res_array[$key]->responseStatus == 200) {
			    		$translated_const_array[$const_index] = str_replace('"', "'", html_entity_decode($res_array[$key]->responseData->translatedText, ENT_QUOTES));
			    	} else {
			    		$translated_const_array[$const_index] = $need_to_translate[2][$key];
			    	}
		    	}

		    	foreach ($translated_const_array AS $const_index=>$const) {
		    		$translation_constants = preg_replace("/language\['$const_index'\] = \"(.*)\";/", "language['$const_index'] = \"$const\";", $translation_constants);
				}
		    	
				/*if (preg_match_all("/{\"translatedText\":\"([^\"]+)\"/i", $response, $response_matches)) {
					foreach ($need_to_translate[1] AS $key=>$const_index) {
						$rm = $response_matches[1][$key];
						$translated_const_array[$const_index] = preg_replace_callback("/\\\u([0-9a-f]{4})/i", create_function('$matches', 'return html_entity_decode(\'&#x\'.$matches[1].\';\', ENT_NOQUOTES, \'UTF-8\');'), $rm);
					}
				}
				foreach ($translated_const_array AS $const_index=>$const) {
		    		$translation_constants = preg_replace("/language\['$const_index'\] = \"(.*)\";/", "language['$const_index'] = \"$const\";", $translation_constants);
				}*/

				$need_to_translate = array();
				$chars_count = 0;
				$i = 0;
    		}
    	}
    	
    	// --------------------------------------------------------------------------------------------
    	// In order to avoid 'MYSQL server has gone away' problem
    	// --------------------------------------------------------------------------------------------
    	$this->db->reconnect();
    	// --------------------------------------------------------------------------------------------

    	return file_put_contents(LANGPATH . $language_code . DIRECTORY_SEPARATOR . $file_name, $translation_constants);
    }
    
    public function translateString($string, $from_lang_code, $to_lang_code)
    {
    	$languagePair = $from_lang_code . "|" . $to_lang_code;
    	$url = "http://ajax.googleapis.com/ajax/services/language/translate?v=1.0&langpair=" . rawurlencode($languagePair);

    	$fullUrl = $url;
    	$fullUrl .= "&q=".rawurlencode($string);
    	
    	$curl_handle=curl_init();
		curl_setopt($curl_handle, CURLOPT_URL, $fullUrl);
		curl_setopt($curl_handle, CURLOPT_CONNECTTIMEOUT, 2);
		curl_setopt($curl_handle, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl_handle, CURLOPT_REFERER, "http://".$_SERVER['HTTP_HOST']);
		$response = curl_exec($curl_handle);
		curl_close($curl_handle);

		if (preg_match_all("/{\"translatedText\":\"([^\"]+)\"/i", $response, $response_matches)) {
			$rm = $response_matches[1];
			$arr = preg_replace_callback("/\\\u([0-9a-f]{4})/i", create_function('$matches', 'return html_entity_decode(\'&#x\'.$matches[1].\';\', ENT_NOQUOTES, \'UTF-8\');'), $rm);
			return $arr[0];
		}
    }
    
    public function copyLanguageFile($default_language_code, $language_code, $file_name)
    {
    	if (@copy(LANGPATH . $default_language_code . '/' . $file_name, LANGPATH . $language_code . '/' . $file_name) &&
    		@chmod(LANGPATH . $language_code . '/' . $file_name, 0777))
    		return true;
    	else 
    		return false;
    }
    
    public function getLanguageById()
	{
		$this->db->select();
		$this->db->from('languages');
		$this->db->where('id', $this->_language_id);
		$query = $this->db->get();
		
		$language = new language;
		$language->setLanguageFromArray($query->row_array());
        return $language;
	}
	
	public function saveLanguageById($form, $current_language)
	{
		$this->db->select('db_code');
		$this->db->from('languages');
		$this->db->where('id', $this->_language_id);
		$query = $this->db->get();
		$row = $query->row_array();
		$old_code = $row['db_code'];
		
    	$this->db->set('active', $form['active']);
		$this->db->set('name', $form['name']);
		if ($current_language->code != 'en') {
			$this->db->set('code', $form['code']);
			$this->db->set('db_code', $form['db_code']);
		}
		$this->db->set('flag', $form['flag']);
		$this->db->set('custom_theme', $form['custom_theme']);
		$this->db->set('decimals_separator', $form['decimals_separator']);
		$this->db->set('thousands_separator', $form['thousands_separator']);
		$this->db->set('date_format', $form['date_format']);
		$this->db->set('time_format', $form['time_format']);
		$this->db->where('id', $this->_language_id);
		if ($this->db->update('languages')) {
			if ($current_language->code != 'en')
				return $this->renameLangColumns($old_code, $form['db_code']);
			return true;
		}
	}
	
	public function saveLanguageDir($new_code)
	{
		$this->db->select('code');
		$this->db->from('languages');
		$this->db->where('id', $this->_language_id);
		$query = $this->db->get();
		
		$row = $query->row_array();
		$old_code = $row['code'];
		
		return rename(LANGPATH . $old_code, LANGPATH . $new_code);
	}
	
	public function deleteLanguageDir()
	{
		$this->db->select('code');
		$this->db->from('languages');
		$this->db->where('id', $this->_language_id);
		$query = $this->db->get();
		
		if ($row = $query->row_array()) {
			$code = $row['code'];
			
			$this->load->helper('directory');
			if ($map = directory_map(LANGPATH . $code)) {
				foreach ($map as $file) {
					unlink(LANGPATH . $code . '/' . $file);
				}
				return rmdir(LANGPATH . $code);
			} else {
				return true;
			}
		}
		return false;
	}
	
	public function deleteLanguageById()
    {
    	$this->db->select('code');
		$this->db->from('languages');
		$this->db->where('id', $this->_language_id);
		$query = $this->db->get();
		if ($row = $query->row_array()) {
			$old_code = $row['code'];
	    	
	        $this->db->where('id', $this->_language_id);
	        if ($this->db->delete('languages')) {
	        	return $this->deleteLangColumns($old_code);
	        }
		}
    }
    
    public function getLanguageFilesList()
    {
    	$this->db->select();
		$this->db->from('language_files');
		$query = $this->db->get();
		
		$lang_files = array();
		foreach ($query->result_array() AS $row) {
			$lang_files[] = $row;
		}
		return $lang_files;
    }
    
    public function getFileNameById($file_id)
    {
    	$this->db->select();
		$this->db->from('language_files');
		$this->db->where('id', $file_id);
		$query = $this->db->get();
		
		$row = $query->row_array();
		return $row['file'];
    }
    
    public function includeLanguageFile($language_code, $file_name)
    {
		include(LANGPATH . $language_code . '/' . $file_name);
		return $language;
    }
    
    public function saveLanguageConstantsIntoFile($language_code, $file_name, $post_values)
    {
    	$file = LANGPATH . $language_code . '/' . $file_name;
    	
    	$translation_constants = file_get_contents($file);
    	foreach ($post_values AS $const_index=>$translated_const) {
    		// replace " -  symbol from constants
    		$translated_const = str_replace('"', "'", $translated_const);
    		// empty constant must contain space
    		if (empty($translated_const))
    			$translated_const = ' ';
    		$translation_constants = preg_replace("/language\['$const_index'\] = \"(.*)\";/", "language['$const_index'] = \"$translated_const\";", $translation_constants);
    	}
    	return file_put_contents($file, $translation_constants);
    }
    
    public function getLanguageByCode($code)
	{
		$this->db->select();
		$this->db->from('languages');
		$this->db->where('code', $code);
		$query = $this->db->get();
		
		$language = new language;
		$language->setLanguageFromArray($query->row_array());
        return $language;
	}
	
	public function getLanguageByDBCode($db_code)
	{
		$this->db->select();
		$this->db->from('languages');
		$this->db->where('db_code', $db_code);
		$query = $this->db->get();
		
		$language = new language;
		$language->setLanguageFromArray($query->row_array());
        return $language;
	}
	
	public function langAreasSwitchOn()
    {
    	$system_settings = registry::get('system_settings');
	    $system_settings['language_areas_enabled'] = 1;
	    registry::set('system_settings', $system_settings);
    }
    
    public function langAreasSwitchOff()
    {
    	$system_settings = registry::get('system_settings');
	    $system_settings['language_areas_enabled'] = 0;
	    registry::set('system_settings', $system_settings);
    }
	
	public function getLangAreaField($lang_code, $table, $field, $row_id, $virtual_id)
    {
    	/*$this->db->execSql('SHOW COLUMNS FROM `' . $table . '` WHERE field = "' . $field . '"');
		$field_props = $this->db->rowAsArray();
    	if (strpos($field_props['Type'], 'varchar') !== false)
    		$field_type = 'varchar';
    	if (strpos($field_props['Type'], 'textarea') !== false)
    		$field_type = 'textarea';*/

		if ($row_id == 'new') {
			if ($this->session->userdata('translation_array')) {
				$translation_array = unserialize(base64_decode($this->session->userdata('translation_array')));
				if (isset($translation_array[$table][$field][$virtual_id][$lang_code]))
					return $translation_array[$table][$field][$virtual_id][$lang_code];
				else
					return 'untranslated';
			}
			return 'untranslated';
		} else {
			$this->langAreasSwitchOff();

		    if ($lang_code != $this->config->item('default_language')) {
		    	$this->db->select($field . ' AS original_value');
		    	$this->db->select($lang_code . '_' . $field . ' AS lang_value');
		    } else {
		    	$this->db->select($field . ' AS lang_value');
		    }
		    $this->db->from($table);
		    $this->db->where('id', $row_id);
		    $query = $this->db->get();

		    $this->langAreasSwitchOn();

		    if ($query->num_rows()) {
				$lang_areas_fields = $query->row_array();
				return $lang_areas_fields['lang_value'];
			} else {
				return false;
			}
		}
    }
    
    public function setLangAreasField($lang_code, $value, $table, $field, $row_id, $virtual_id = 0)
    {
    	if ($row_id == 'new') {
    		// Save translated value in session
    		if ($this->session->userdata('translation_array')) {
				$translation_array = unserialize(base64_decode($this->session->userdata('translation_array')));
    		}
			$translation_array[$table][$field][$virtual_id][$lang_code] = $value;
			$this->session->set_userdata(array('translation_array' => base64_encode(serialize($translation_array))));
			return true;
    	} else {
    		// Or in database
    		$this->langAreasSwitchOff();
    		
    		if ($lang_code != $this->config->item('default_language')) {
	    		$lang_field = $lang_code . '_' . $field;
    		} else {
    			$lang_field = $field;
    		}

			$this->db->set($lang_field, $value);
	    	$this->db->where('id', $row_id);
	    	$result = $this->db->update($table);

	    	$this->langAreasSwitchOn();

	    	return $result;
    	}
    }
    
    public function addLangColumns($code, $i18n_fields = null)
	{
		if (is_null($i18n_fields)) {
			$i18n_fields = registry::get('i18n_fields');
		}
		
		$tables = $this->db->list_tables();
		foreach ($i18n_fields AS $table=>$fields) {
			if (!in_array($table, $tables)) {
				unset($i18n_fields[$table]);
			}
		}

		foreach ($i18n_fields AS $table=>$fields) {
			foreach ($fields AS $field) {
				$new_field = $code . '_' . $field;
				$query = $this->db->query('SHOW COLUMNS FROM `' . $table . '` WHERE field = "' . $new_field . '"');
				if (!count($query->row_array())) {
					$query = $this->db->query('SHOW COLUMNS FROM `' . $table . '` WHERE field = "' . $field . '"');
					$field_props = $query->row_array();
					
					if ($field_props['Type'] == 'text') {
						$default = '';
					} else {
						$default = 'DEFAULT "untranslated" NOT NULL';
					}

					$this->db->query('ALTER TABLE `' . $table . '` ADD ' . $new_field . ' ' . $field_props['Type'] . ' ' . $default);
				}
			}
		}
		return true;
	}
	
	public function renameLangColumns($old_code, $new_code)
	{
		$i18n_fields = registry::get('i18n_fields');
		
		foreach ($i18n_fields AS $table=>$fields) {
			foreach ($fields AS $field) {
				$query = $this->db->query('SHOW COLUMNS FROM `' . $table . '` WHERE field = "' . $field . '"');
				$field_props = $query->row_array();
				
				if ($field_props['Type'] == 'text') {
					$default = '';
				} else {
					$default = 'DEFAULT "untranslated" NOT NULL';
				}
				
				$old_field = $old_code . '_' . $field;
				$new_field = $new_code . '_' . $field;
				$this->db->query('ALTER TABLE `' . $table . '` CHANGE ' . $old_field . ' ' . $new_field .' ' . $field_props['Type'] . ' ' . $default);
			}
		}
		return true;
	}
	
	public function deleteLangColumns($code)
	{
		$i18n_fields = registry::get('i18n_fields');

		$this->load->dbforge();

		foreach ($i18n_fields AS $table=>$fields) {
			$fields = array_unique($fields);
			foreach ($fields AS $field) {
				$delete_field = $code . '_' . $field;
				if ($this->db->field_exists($delete_field, $table)) {
					$this->dbforge->drop_column($table, $delete_field);
				}
			}
		}
		return true;
	}
}
?>