<?php
include_once(MODULES_PATH . 'listings/classes/listing.class.php');
include_once(MODULES_PATH . 'listings/classes/listing_image.class.php');
include_once(MODULES_PATH . 'listings/classes/listing_file.class.php');
include_once(MODULES_PATH . 'listings/classes/listing_location.class.php');
include_once(MODULES_PATH . 'google_maps/classes/location_geoname.class.php');
include_once(MODULES_PATH . 'listings/classes/search_core.class.php');
include_once(MODULES_PATH . 'notifications/classes/notification_sender.class.php');
include_once(MODULES_PATH . 'categories/classes/category.class.php');

class listingsModel extends model
{
    private $_listing_id;
    private $_cached_listings = array();

    public function setListingId($listing_id)
    {
    	$this->_listing_id = $listing_id;
    }

    /**
     * get listing row by seo name or ID
     *
     * @param string/integer $url_part
     * @return array
     */
    public function getListingRowByUrl($url_part)
    {
    	$this->db->select();
    	$this->db->from('listings');
    	if (is_numeric($url_part))
    		$this->db->where('id', $url_part);
    	else
    		$this->db->where('seo_title', $url_part);
    	return $this->db->get()->row_array();
    }

    public function getLevelIdByListingId($listing_id = null)
    {
    	if (is_null($listing_id))
    		$listing_id = $this->_listing_id;

    	$this->db->select('level_id');
    	$this->db->from('listings');
    	$this->db->where('id', $listing_id);
    	$query = $this->db->get();
    	
    	if ($query->num_rows) {
	    	$row = $query->row_array();
	    	return $row['level_id'];
    	} else 
    		return false;
    }

    /**
     * is there listing with such seoname in the DB?
     *
     * @param string $seoname
     */
    public function is_listing_seo_name($seoname)
    {
    	$this->db->select();
		$this->db->from('listings');
		$this->db->where('seo_title', $seoname);
		if (!is_null($this->_listing_id)) {
			$this->db->where('id !=', $this->_listing_id);
		}
		$query = $this->db->get();

		return $query->num_rows();
    }
    
    public function saveListing($form, $level, $type, $user_package_id = null)
    {
    	if ($level) {
	    	$this->db->set('level_id', $level->id);
	    	$this->db->set('owner_id', $this->session->userdata('user_id'));

	    	if ($level->title_enabled)
	    		$this->db->set('title', $form['name']);
	    	if ($level->seo_title_enabled)
	    		$this->db->set('seo_title', $form['seo_name']);
	    	if ($level->description_mode != 'disabled')
	    		$this->db->set('listing_description', $form['listing_description']);
	    	if ($level->meta_enabled) {
				$this->db->set('listing_meta_description', $form['listing_meta_description']);
				$this->db->set('listing_keywords', str_replace("\n", ", ", $form['listing_keywords']));
	    	}
	    	if (isset($form['listing_logo_image']))
	    		$this->db->set('logo_file', $form['listing_logo_image']);
	    	$this->db->set('last_modified_date', date("Y-m-d H:i:s"));
	
	    	if ($level->preapproved_mode)
	    		$this->db->set('status', 4); // unapproved
	    	else
	    		$this->db->set('status', 1); // active
	
	    	$this->db->set('creation_date', date("Y-m-d H:i:s"));
	    	$this->db->set('last_modified_date', date("Y-m-d H:i:s"));
	    	// Set custom expiration date if user have permission and listing not under eternal active period
	    	$content_access_obj = contentAcl::getInstance();
	    	if ($content_access_obj->isPermission('Edit listings expiration date') && !$level->eternal_active_period) {
	    		$this->db->set('expiration_date', date("Y-m-d H:i:s", strtotime($form['expiration_date'])));
	    	} else {
	    		$this->db->set('expiration_date', date("Y-m-d H:i:s", (mktime() + (
	    											($level->active_days) +
	    											($level->active_months*30) +
	    											($level->active_years*365)
	    											)*86400)));
	    	}
	
	    	$this->db->set('was_prolonged_times', '0');
	    	
	    	if ($type->categories_type != 'disabled' && $level->categories_number) {
		    	$categories_array = unserialize($form['serialized_categories_list']);
				$categories_array = array_filter($categories_array);
	    	}

	    	if (!$type->locations_enabled || ($type->locations_enabled && count($form['location_id[]']) <= $level->locations_number)
	    	&& count($categories_array) <= $level->categories_number) {
	    		if ($this->db->insert('listings')) {
		    		$listing_id = $this->db->insert_id();
		    		
		    		$system_settings = registry::get('system_settings');
		        	if (isset($system_settings['language_areas_enabled']) && $system_settings['language_areas_enabled']) {
		        		translations::saveTranslations(array('listings', 'title', $listing_id));
		        		translations::saveTranslations(array('listings', 'listing_description', $listing_id));
		        	}
		        	
		        	if (!is_null($user_package_id)) {
		        		$this->db->set('user_package_id', $user_package_id);
		        		$this->db->set('listing_id', $listing_id);
		        		$this->db->insert('packages_listings');
		        	}
		        	
		        	// --------------------------------------------------------------------------------------------
		        	// Does this listing can be claimed?
		        	// --------------------------------------------------------------------------------------------
		        	$content_access_obj = contentAcl::getInstance();
			    	if ($content_access_obj->isPermission('Manage ability to claim')) {
			    		if ($form['ability_to_claim']) {
			    			$this->db->set('listing_id', $listing_id);
			    			$this->db->set('ability_to_claim', 1);
			    			$this->db->set('from_user_id', $this->session->userdata('user_id'));
			    			$this->db->insert('listings_claims');
			    		}
			    	}
			    	// --------------------------------------------------------------------------------------------
	
			    	if ($type->locations_enabled && $level->locations_number) {
			    		$CI = &get_instance();
						$CI->load->model('locations', 'locations_predefined');
						$locations_levels = $CI->locations->selectAllLevels();

						foreach ($form['location_id[]'] AS $key=>$virtual_location_id) {
							// Insert new location

							$use_predefined_locations = 0;
							if ($system_settings['predefined_locations_mode'] != 'disabled') {
								// Save predefined location
								$lastlevel_loc_id = 0;
								foreach ($locations_levels AS $locations_level)
									if ($form['loc_level_' . $locations_level->order_num . '[]'][$key])
										$lastlevel_loc_id = $form['loc_level_' . $locations_level->order_num . '[]'][$key];
								if ($lastlevel_loc_id) {
									$this->db->set('predefined_location_id', $lastlevel_loc_id);
	
									if ($system_settings['predefined_locations_mode'] == 'only' || (is_array($form['use_predefined_locations']) && in_array($form['location_id[]'][$key], $form['use_predefined_locations']))) {
										$use_predefined_locations = 1;
										$location = $CI->locations->getLocationById($lastlevel_loc_id);
								    	$geocoder = new locationGeoname;
								    	if ($geocoded_name = $geocoder->geonames_request($location->getChainAsString()))
								    		$this->db->set('geocoded_name', $geocoded_name);
									}
									$this->db->set('use_predefined_locations', $use_predefined_locations);
								}
							}

							if ($system_settings['predefined_locations_mode'] != 'only') {
								// Save location string
								if ($form['location[]'][$key] && !$use_predefined_locations) {
									$this->db->set('location', $form['location[]'][$key]);
									$geocoder = new locationGeoname;
									if ($geocoded_name = $geocoder->geonames_request($form['location[]'][$key]))
										$this->db->set('geocoded_name', $geocoded_name);
								}
							}
							
							if ($use_predefined_locations || ($system_settings['predefined_locations_mode'] != 'only' && $form['location[]'][$key])) {
								$this->db->set('listing_id', $listing_id);
								$this->db->set('address_line_1', $form['address_line_1[]'][$key]);
								$this->db->set('address_line_2', $form['address_line_2[]'][$key]);
								if ($type->zip_enabled)
									$this->db->set('zip_or_postal_index', $form['zip_or_postal_index[]'][$key]);
								if ($level->maps_enabled && @$system_settings['maps_key']) {
									if (is_array($form['manual_coords']) && in_array($form['location_id[]'][$key], $form['manual_coords']))
										$manual = 1;
									else 
										$manual = 0;
									$this->db->set('manual_coords', $manual);
									$this->db->set('map_coords_1', $form['map_coords_1[]'][$key]);
									$this->db->set('map_coords_2', $form['map_coords_2[]'][$key]);
									$this->db->set('map_zoom', $form['map_zoom[]'][$key]);
									$this->db->set('map_icon_id', $form['map_icon_id[]'][$key]);
									$this->db->set('map_icon_file', $form['map_icon_file[]'][$key]);
								}
								$this->db->insert('listings_in_locations');
								$location_id = $this->db->insert_id();
								// Save locations translations using virtual IDs
								if (isset($system_settings['language_areas_enabled']) && $system_settings['language_areas_enabled']) {
									translations::saveTranslations(array('listings_in_locations', 'location', $location_id, $virtual_location_id));
									translations::saveTranslations(array('listings_in_locations', 'address_line_1', $location_id, $virtual_location_id));
									translations::saveTranslations(array('listings_in_locations', 'address_line_2', $location_id, $virtual_location_id));
								}
							}
						}
			    	}

			    	if ($type->categories_type != 'disabled' && $level->categories_number) {
						foreach ($categories_array AS $category_id) {
							$this->db->set('listing_id', $listing_id);
							$this->db->set('category_id', $category_id);
							$this->db->insert('listings_in_categories');
						}
			    	}
	
		    		return $listing_id;
	    		} else {
		    		return false;
		    	}
	    	} else {
	    		return false;
	    	}
    	} else {
    		return false;
    	}
    }
    
    public function saveListingById($form, $level, $type, $existed_listing_obj)
    {
    	$system_settings = registry::get('system_settings');
    	
    	$content_access_obj = contentAcl::getInstance();

    	if ($level->title_enabled)
	    	$this->db->set('title', $form['name']);
	    if ($level->seo_title_enabled)
	    	$this->db->set('seo_title', $form['seo_name']);
	    if ($level->description_mode != 'disabled')
	    	$this->db->set('listing_description', $form['listing_description']);
	    if ($level->meta_enabled) {
			$this->db->set('listing_meta_description', $form['listing_meta_description']);
			$this->db->set('listing_keywords', str_replace("\n", ", ", $form['listing_keywords']));
	    }
		if (isset($form['listing_logo_image']))
    		$this->db->set('logo_file', $form['listing_logo_image']);
    	$this->db->set('last_modified_date', date("Y-m-d H:i:s"));
    	// Set custom expiration date if user have permission and listing not under eternal active period
    	if ($content_access_obj->isPermission('Edit listings expiration date') && !$level->eternal_active_period) {
    		$this->db->set('expiration_date', date("Y-m-d H:i:s", strtotime($form['expiration_date'])));
    	}
    	$this->db->where('id', $this->_listing_id);

    	if ($type->categories_type != 'disabled' && $level->categories_number) {
	    	$categories_array = unserialize($form['serialized_categories_list']);
			$categories_array = array_filter($categories_array);
    	}

    	if (!$type->locations_enabled || ($type->locations_enabled && count($form['location_id[]']) <= $level->locations_number)
	    && count($categories_array) <= $level->categories_number) {
	    	if ($this->db->update('listings')) {
	    		if ($type->locations_enabled && $level->locations_number) {
	    			$CI = &get_instance();
					$CI->load->model('locations', 'locations_predefined');
					$locations_levels = $CI->locations->selectAllLevels();

					// Save listings in locations
					$actual_locations_array = array();
					$loc_array = $this->getListingLocations();
					foreach ($loc_array AS $location)
						$actual_locations_array[] = $location->id;
	
					foreach ($form['location_id[]'] AS $key=>$virtual_location_id) {
						if (!in_array($virtual_location_id, $actual_locations_array)) {
							// Insert new location

							$use_predefined_locations = 0;
							if ($system_settings['predefined_locations_mode'] != 'disabled') {
								// Save predefined location
								$lastlevel_loc_id = 0;
								foreach ($locations_levels AS $locations_level)
									if ($form['loc_level_' . $locations_level->order_num . '[]'][$key])
										$lastlevel_loc_id = $form['loc_level_' . $locations_level->order_num . '[]'][$key];
								if ($lastlevel_loc_id) {
									$this->db->set('predefined_location_id', $lastlevel_loc_id);
	
									if ($system_settings['predefined_locations_mode'] == 'only' || (is_array($form['use_predefined_locations']) && in_array($form['location_id[]'][$key], $form['use_predefined_locations']))) {
										$use_predefined_locations = 1;
										$location = $CI->locations->getLocationById($lastlevel_loc_id);
								    	$geocoder = new locationGeoname;
								    	if ($geocoded_name = $geocoder->geonames_request($location->getChainAsString()))
								    		$this->db->set('geocoded_name', $geocoded_name);
									}
									$this->db->set('use_predefined_locations', $use_predefined_locations);
								}
							}

							if ($system_settings['predefined_locations_mode'] != 'only') {
								// Save location string
								if ($form['location[]'][$key] && !$use_predefined_locations) {
									$this->db->set('location', $form['location[]'][$key]);
									$geocoder = new locationGeoname;
									if ($geocoded_name = $geocoder->geonames_request($form['location[]'][$key]))
										$this->db->set('geocoded_name', $geocoded_name);
								}
							}

							if ($use_predefined_locations || ($system_settings['predefined_locations_mode'] != 'only' && $form['location[]'][$key])) {
								$this->db->set('listing_id', $this->_listing_id);
								$this->db->set('address_line_1', $form['address_line_1[]'][$key]);
								$this->db->set('address_line_2', $form['address_line_2[]'][$key]);
								if ($type->zip_enabled)
									$this->db->set('zip_or_postal_index', $form['zip_or_postal_index[]'][$key]);
								if ($level->maps_enabled && @$system_settings['maps_key']) {
									if (is_array($form['manual_coords']) && in_array($form['location_id[]'][$key], $form['manual_coords']))
										$manual = 1;
									else 
										$manual = 0;
									$this->db->set('manual_coords', $manual);
									$this->db->set('map_coords_1', $form['map_coords_1[]'][$key]);
									$this->db->set('map_coords_2', $form['map_coords_2[]'][$key]);
									$this->db->set('map_zoom', $form['map_zoom[]'][$key]);
									$this->db->set('map_icon_id', $form['map_icon_id[]'][$key]);
									$this->db->set('map_icon_file', $form['map_icon_file[]'][$key]);
								}
								$this->db->insert('listings_in_locations');
								$location_id = $this->db->insert_id();
								// Save locations translations using virtual IDs
						        if (isset($system_settings['language_areas_enabled']) && $system_settings['language_areas_enabled']) {
						        	translations::saveTranslations(array('listings_in_locations', 'location', $location_id, $virtual_location_id));
						        	translations::saveTranslations(array('listings_in_locations', 'address_line_1', $location_id, $virtual_location_id));
						        	translations::saveTranslations(array('listings_in_locations', 'address_line_2', $location_id, $virtual_location_id));
								}
							}
						} else {
							// Update existed location
							$location_id = $virtual_location_id;

							$use_predefined_locations = 0;
							if ($system_settings['predefined_locations_mode'] != 'disabled') {
								// Save predefined location
								$lastlevel_loc_id = 0;
								foreach ($locations_levels AS $locations_level)
									if ($form['loc_level_' . $locations_level->order_num . '[]'][$key])
										$lastlevel_loc_id = $form['loc_level_' . $locations_level->order_num . '[]'][$key];
								if ($lastlevel_loc_id) {
									$this->db->set('predefined_location_id', $lastlevel_loc_id);
	
									if ($system_settings['predefined_locations_mode'] == 'only' || (is_array($form['use_predefined_locations']) && in_array($form['location_id[]'][$key], $form['use_predefined_locations']))) {
										$use_predefined_locations = 1;
										$location = $CI->locations->getLocationById($lastlevel_loc_id);
								    	$geocoder = new locationGeoname;
								    	if ($geocoded_name = $geocoder->geonames_request($location->getChainAsString()))
								    		$this->db->set('geocoded_name', $geocoded_name);
									}
									$this->db->set('use_predefined_locations', $use_predefined_locations);
								}
								
							}

							if ($system_settings['predefined_locations_mode'] != 'only') {
								// Save location string
								if ($form['location[]'][$key] && !$use_predefined_locations) {
									$this->db->set('location', $form['location[]'][$key]);
									$geocoder = new locationGeoname;
									if ($geocoded_name = $geocoder->geonames_request($form['location[]'][$key]))
										$this->db->set('geocoded_name', $geocoded_name);
								}
							}

							$this->db->set('address_line_1', $form['address_line_1[]'][$key]);
							$this->db->set('address_line_2', $form['address_line_2[]'][$key]);
							if ($type->zip_enabled)
								$this->db->set('zip_or_postal_index', $form['zip_or_postal_index[]'][$key]);
							if ($level->maps_enabled && @$system_settings['maps_key']) {
								if (is_array($form['manual_coords']) && in_array($form['location_id[]'][$key], $form['manual_coords']))
									$manual = 1;
								else 
									$manual = 0;
								$this->db->set('manual_coords', $manual);
								$this->db->set('map_coords_1', $form['map_coords_1[]'][$key]);
								$this->db->set('map_coords_2', $form['map_coords_2[]'][$key]);
								$this->db->set('map_zoom', $form['map_zoom[]'][$key]);
								$this->db->set('map_icon_id', $form['map_icon_id[]'][$key]);
								$this->db->set('map_icon_file', $form['map_icon_file[]'][$key]);
							}
							$this->db->where('id', $location_id);
							$this->db->update('listings_in_locations');
							$keys = array_keys($actual_locations_array, $location_id);
							unset($actual_locations_array[$keys[0]]);
						}
					}
					// Delete locations those were unchecked
					foreach ($actual_locations_array AS $location_id) {
						$this->db->delete('listings_in_locations', array('id' => $location_id));
					}
	    		}
	
				// --------------------------------------------------------------------------------------------
		        // Does this listing can be claimed?
		        // --------------------------------------------------------------------------------------------
			    if ($content_access_obj->isPermission('Manage ability to claim')) {
			    	if ($form['ability_to_claim']) {
			    		$this->db->set('listing_id', $this->_listing_id);
			    		$this->db->set('ability_to_claim', 1);
			    		$this->db->set('from_user_id', $existed_listing_obj->owner_id);
			    		$this->db->on_duplicate_insert('listings_claims');
			    	} else {
			    		$this->db->delete('listings_claims', array('listing_id' => $this->_listing_id));
			    	}
			    }
			    // --------------------------------------------------------------------------------------------


			    if ($type->categories_type != 'disabled' && $level->categories_number) {
					// Save listings in categories
					$actual_categories = $this->getListingCategories();
					$actual_categories_array = array();
					foreach ($actual_categories AS $category)
						$actual_categories_array[] = $category->id;
					$refresh_cache = false;
					foreach ($categories_array AS $category_id) {
						if (!in_array($category_id, $actual_categories_array)) {
							$this->db->set('listing_id', $this->_listing_id);
							$this->db->set('category_id', $category_id);
							$this->db->on_duplicate_insert('listings_in_categories');
							$refresh_cache = true;
						} else {
							$keys = array_keys($actual_categories_array, $category_id);
							unset($actual_categories_array[$keys[0]]);
						}
					}

					// Delete categories those were unchecked
					foreach ($actual_categories_array AS $category_id) {
						$this->db->delete('listings_in_categories', array('listing_id' => $this->_listing_id, 'category_id' => $category_id));
						$refresh_cache = true;
					}
			    }
				return true;
	    	} else {
	    		return false;
	    	}
        } else {
        	return false;
        }
    }
    
    /**
     * Saves listing level
     *
     * @param int $level_id
     * @return bool
     */
    public function saveListingLevelAndExpirationDate($level_id, $new_time)
    {
    	$this->db->set('level_id', $level_id);
    	$this->db->set('expiration_date', date("Y-m-d H:i:s", $new_time));
    	$this->db->set('last_modified_date', date("Y-m-d H:i:s"));
    	$this->db->where('id', $this->_listing_id);
    	return $this->db->update('listings');
    }

    /**
     * Saves listing status
     *
     * @param int $status
     * @return bool
     */
    public function saveListingStatus($status)
    {
    	$this->db->set('status', $status);
    	$this->db->where('id', $this->_listing_id);
    	return $this->db->update('listings');
    }
    
    /**
     * select recursively categories childs
     *
     * @param array $categories
     * @param int $parent_id
     * @param array $ids
     */
    public function getChildCategoriesIdsRecursive($categories, $parent_id, &$ids)
    {
    	foreach ($categories AS $category) {
    		if ($category['parent_category_id'] == $parent_id) {
    			$ids[] = $category['id'];
    			$this->getChildCategoriesIdsRecursive($categories, $category['id'], $ids);
    		}
    	}
    }

    /**
     * Select all listings table using paginator,
     * this executes optimized method with 3 queries
     *
     * @return array
     */
    public function selectListings($args = array(), $orderby = 'id', $direction = 'desc', $search_fields_sql_array = array(), $categories_db = array(), $fully_load = true)
    {
    	$CI = &get_instance();

    	if (isset($args['page']) && $args['page'])
    		$page = $args['page'];
    	else 
    		$page = null;
    	
    	$current_type = null;
    	if (isset($args['search_type']) && $args['search_type']) {
    		if (is_numeric($args['search_type'])) {
    			$CI->load->model('types', 'types_levels');
    			$current_type = $CI->types->getTypeById($args['search_type']);
    		} elseif (is_object($args['search_type'])) {
    			$current_type = $args['search_type'];
    		} elseif (is_string($args['search_type'])) {
    			$CI->load->model('types', 'types_levels');
    			$current_type = $CI->types->getTypeBySeoName($args['search_type']);
    		}
    	}
    	
    	if (isset($args['only_with_logos']) && ($args['only_with_logos'] === TRUE || $args['only_with_logos'] === 'true')) {
    		$only_with_logos = true;
    	} else {
    		$only_with_logos = false;
    	}
    	
    	$search_owner_id = null;
    	$search_owner_login = null;
    	$search_owners = null;
    	if (isset($args['search_owner']) && $args['search_owner']) {
    		if (is_object($args['search_owner'])) {
    			$search_owner = $args['search_owner'];
    			$search_owner_id = $search_owner->id;
    		} elseif (is_array($args['search_owner'])) {
    			$search_owners = $args['search_owner'];
    		} elseif (is_numeric($args['search_owner'])) {
    			$search_owner_id = $args['search_owner'];
    		} elseif (is_string($args['search_owner'])) {
    			$search_owner_login = urldecode(html_entity_decode($args['search_owner']));
    		}
    	}

    	// --------------------------------------------------------------------------------------------
		// If we need to order by content field value - prepare its type and ID
		if (strpos($orderby, 'cf.') !== FALSE) {
			$this->db->select('cf.type AS type');
			$this->db->select('cf.id AS id');
			$this->db->from('content_fields AS cf');
			$this->db->where('cf.seo_name', substr($orderby, 3));
			$row = $this->db->get()->row_array();
    		$content_field_type = $row['type'];
    		$content_field_id = $row['id'];
		}
		// --------------------------------------------------------------------------------------------

		if ($page == 'search' && !isset($args['where_radius'])) {
			$search_core = new searchCore;
			$search_core->setArgs($args);
			$what_listings_ids = $search_core->processWhat();
			$where_listings_ids = $search_core->processWhere();
		} else {
			$what_listings_ids = array();
			$where_listings_ids = array();
		}

    	// Radius search
    	if (isset($args['where_radius']) && $args['where_radius'] && isset($args['where_search']) && $args['where_search']) {
    		$where_radius = $args['where_radius'];
    		
    		$system_settings = registry::get('system_settings');
    		if ($system_settings['search_in_raduis_measure'] == 'miles') {
    			$miles = pow($where_radius*0.018, 2);
    			$measure = $miles;
    		} else {
    			$kilometres = pow($where_radius*0.011, 2);
    			$measure = $kilometres;
    		}

    		$geocoder = new locationGeoname;
    		if ($coordinates = $geocoder->geonames_request($args['where_search'], 'coordinates')) {
	    		$x = $coordinates[1];
	    		$y = $coordinates[0];
	    		
	    		$this->db->select('lil.listing_id');
		    	$this->db->from('listings_in_locations as lil');
		    	// (x0-x1)*(x0-x1)+(y0-y1)*(y0-y1) = R*R
		    	$this->db->or_where('POWER((lil.map_coords_1-' . $x . '), 2)+POWER((lil.map_coords_2-' . $y . '), 2) <= ' . $measure);
	
				$where_radius_ids = str_replace("\n", " ", $this->db->_compile_select());
				$this->db->_reset_select();
    		}
    	}

    	if (isset($args['search_category']) && $args['search_category']) {
    		if (is_array($args['search_category'])) {
    			$top_categories_ids = $args['search_category'];
    		} elseif (is_numeric($args['search_category'])) {
    			$top_categories_ids = array($args['search_category']);
    		} elseif (is_string($args['search_category'])) {
    			$top_categories_ids = array_values(array_filter(explode(',', urldecode(html_entity_decode($args['search_category'])))));
    			if (count($top_categories_ids) == 1 && !is_numeric($top_categories_ids[0])) {
    				$CI->load->model('categories', 'categories');
    				$category = $CI->categories->getCategoryBySeoName($top_categories_ids[0]);
    				$top_categories_ids = array($category->id);
    			}
    		} elseif (is_object($args['search_category'])) {
    			$top_categories_ids = array($args['search_category']->id);
    		}
			array_walk($top_categories_ids, "trim");

			// Select categories, if they weren't passed into function
			if (!$categories_db) {
				$CI->load->model('categories', 'categories');
				if (!is_null($current_type)) {
					if ($current_type->categories_type == 'local')
						$CI->categories->setTypeId($current_type->id);
				}
				$categories_db = $CI->categories->selectCategoriesFromDB();
			}
			$categories_ids = array();
			foreach ($top_categories_ids AS $id)
    			$this->getChildCategoriesIdsRecursive($categories_db, $id, $categories_ids);
    		$categories_ids = array_merge($top_categories_ids, $categories_ids);
    	} else {
    		$categories_ids = null;
    	}
    	
    	if (isset($args['search_location']) && $args['search_location']) {
    		if (is_numeric($args['search_location'])) {
    			// By ID
    			$CI = &get_instance();
    			$CI->load->model('locations', 'locations_predefined');
    			$search_location = $CI->locations->getLocationById($args['search_location']);
    		} elseif (is_object($args['search_location']))
    			// This is ready object
    			$search_location = $args['search_location'];
    		else {
    			// By seo name
    			$CI = &get_instance();
    			$CI->load->model('locations', 'locations_predefined');
    			$search_location = $CI->locations->getLocationBySeoName($args['search_location']);
    		}
    	} else {
    		$search_location = null;
    	}
    	
    	if (isset($args['search_by_ids']) && $args['search_by_ids']) {
    		if (is_numeric($args['search_by_ids'])) {
    			$search_by_ids = array($args['search_by_ids']);
    		} elseif (!is_array($args['search_by_ids'])) {
    			$search_by_ids = explode(',', $args['search_by_ids']);
    		} else {
    			$search_by_ids = $args['search_by_ids'];
    		}
    	} else {
    		$search_by_ids = null;
    	}
    	
    	if (isset($args['except_listings']) && $args['except_listings']) {
    		if (is_numeric($args['except_listings'])) {
    			$except_listings = array($args['except_listings']);
    		} elseif (!is_array($args['except_listings'])) {
    			$except_listings = explode(',', $args['except_listings']);
    		} else {
    			$except_listings = $args['except_listings'];
    		}
    	} else {
    		$except_listings = null;
    	}
    	
    	if (isset($args['search_levels']) && $args['search_levels']) {
		   	if (is_numeric($args['search_levels'])) {
    			$search_levels = array($args['search_levels']);
		   	} elseif (is_array($args['search_levels'])) {
		   		$search_levels = $args['search_levels'];
		   	} elseif (is_string($args['search_levels'])) {
		   		$search_levels = explode(',', $args['search_levels']);
		   	} else {
		   		$search_levels = null;
		   	}
	    } else {
	    	$search_levels = null;
	    }
    	// --------------------------------------------------------------------------------------------
    	// --------------------------------------------------------------------------------------------
    	// --------------------------------------------------------------------------------------------
    	// --------------------------------------------------------------------------------------------

    	// --------------------------------------------------------------------------------------------
    	if (isset($this->paginator) && $this->paginator) {
	    	// Number of rows needs
	    	$this->db->select('count(DISTINCT l.id) as count_rows');
	    	$this->db->from('listings as l');
	    	$this->db->join('levels as lev', 'lev.id=l.level_id', 'left');
	    	$this->db->join('types as t', 't.id=lev.type_id', 'left');
	    	$this->db->join('users as u', 'l.owner_id=u.id', 'left');
	    	//$this->db->where_not_in('l.title', 'untranslated'); // except untranslated listings
	
	    	if (isset($args['what_search']) && $what_listings_ids) {
	    		$this->db->where_in('l.id', $what_listings_ids, false);
	    	}
	
	    	if (isset($args['where_search']) && $where_listings_ids) {
	    		$this->db->where_in('l.id', $where_listings_ids, false);
	    	}
	
	    	if (isset($where_radius_ids) && $where_radius_ids) {
	    		$this->db->where_in('l.id', $where_radius_ids, false);
	    	}
	    	
	    	if ($only_with_logos) {
	    		$this->db->where('lev.logo_enabled', 1);
	    		$this->db->not_where('l.logo_file', '');
	    	}
	    	
	    	if (isset($args['search_claimed_listings']) && $args['search_claimed_listings']) {
	    		if ($args['search_claimed_listings'] != 'any') {
	    			$this->db->join('listings_claims AS lc', 'lc.listing_id=l.id', 'left');
	    			if ($args['search_claimed_listings'] == 'ability_to_claim') {
	    				$this->db->where('lc.ability_to_claim', 1);
	    				$this->db->where('lc.approved', 0);
	    			} elseif ($args['search_claimed_listings'] == 'claimed') {
	    				$this->db->where('lc.ability_to_claim', 0);
	    				$this->db->where('lc.approved', 0);
	    				$this->db->where('lc.to_user_id !=', 0);
	    			} elseif ($args['search_claimed_listings'] == 'approved_claim') {
	    				$this->db->where('lc.to_user_id !=', 0);
	    				$this->db->where('lc.approved', 1);
	    			}
	    		}
	    	}
	
	    	if ($current_type) {
	    		$this->db->where('t.id', $current_type->id);
	    	}
	    	if ($page == 'search') {
	    		if ($current_type && $current_type->search_type == 'local') {
		    		$this->db->where('t.search_type', 'local');
		    	} elseif ($current_type && $current_type->search_type == 'global') {
		    		$this->db->where('t.search_type', 'global');
		    	}
		    	$this->db->where('t.search_type !=', 'disabled');
	    	}
	    	
	    	if ($search_levels) {
		    	$this->db->where_in('lev.id', $search_levels);
	    	}
	
	    	if ($categories_ids) {
	    		$this->db->join('listings_in_categories as lic', 'l.id=lic.listing_id', 'left');
	    		$this->db->where_in('lic.category_id', $categories_ids);
	    		
	    		if (is_null($current_type) || $current_type->categories_type == 'global') {
	    			$this->db->where('t.categories_type', 'global');
	    		}
	    	}
	
	    	if ($search_owner_id || $search_owner_login || $search_owners) {
	    		if ($search_owner_id)
	    			$this->db->where('u.id', $search_owner_id);
	    		if ($search_owner_login)
	    			$this->db->like('u.login', $search_owner_login);
	    		if ($search_owners)
	    			$this->db->where_in('u.id', $search_owners);
	    	}
	
	    	if (isset($args['search_status']) && $args['search_status']) {
	    		$this->db->where('l.status', urldecode($args['search_status']));
	    	}
	    	
	    	if (isset($args['search_users_status']) && $args['search_users_status']) {
	    		$this->db->where('u.status', urldecode($args['search_users_status']));
	    	}
	
	    	if (isset($args['search_creation_date']) && $args['search_creation_date']) {
	    		// Receives date in unix timestamp and also in 'Y-m-d' formats
	    		if (($tmstmp = strtotime($args['search_creation_date'])) !== FALSE && strtotime($args['search_creation_date']) != -1)
	    			$this->db->where('TO_DAYS(l.creation_date) = ', 'TO_DAYS("' . date("Y-m-d", $tmstmp) . '")', false);
	    		else
	    			$this->db->where('TO_DAYS(l.creation_date) = ', 'TO_DAYS("' . date("Y-m-d", $args['search_creation_date']) . '")', false);
	    	}
	    	if (isset($args['search_from_creation_date']) && $args['search_from_creation_date']) {
	    		// Receives date in unix timestamp and also in 'Y-m-d' formats
	    		if (($tmstmp = strtotime($args['search_from_creation_date'])) !== FALSE && strtotime($args['search_from_creation_date']) != -1)
	    			$this->db->where('TO_DAYS(l.creation_date) >= ', 'TO_DAYS("' . date("Y-m-d", $tmstmp) . '")', false);
	    		else
	    			$this->db->where('TO_DAYS(l.creation_date) >= ', 'TO_DAYS("' . date("Y-m-d", $args['search_from_creation_date']) . '")', false);
	    		
	    	}
	    	if (isset($args['search_to_creation_date']) && $args['search_to_creation_date']) {
	    		// Receives date in unix timestamp and also in 'Y-m-d' formats
	    		if (($tmstmp = strtotime($args['search_from_creation_date'])) !== FALSE && strtotime($args['search_from_creation_date']) != -1)
	    			$this->db->where('TO_DAYS(l.creation_date) <= ', 'TO_DAYS("' . date("Y-m-d", $tmstmp) . '")', false);
	    		else
	    			$this->db->where('TO_DAYS(l.creation_date) <= ', 'TO_DAYS("' . date("Y-m-d", $args['search_to_creation_date']) . '")', false);
	    	}
	
	    	// Get listings ids from results queries of content pages
	    	foreach ($search_fields_sql_array AS $search_sql) {
	    		if (is_array($search_sql)) {
		    		$condition = key($search_sql);
		    		$sql = $search_sql[$condition];
		    		if ($condition == 'AND')
		    			$this->db->where_in('l.id', $sql, false);
		    		elseif ($condition == 'OR')
		    			$this->db->or_where_in('l.id', $sql, false);
	    		}
	    	}
	    	
	    	if ($search_location) {
	    		$this->db->join('listings_in_locations AS lil', 'lil.listing_id=l.id', 'left');
	    		/*$this->db->like('lil.geocoded_name', $search_location->geocoded_name, 'before');
	    		$this->db->where('t.locations_enabled', true);
	    		$this->db->where('lev.locations_number > 0');*/
	    		$this->db->where('((lil.geocoded_name LIKE "%' . $search_location->geocoded_name . '" AND t.locations_enabled=1 AND lev.locations_number>0) OR lil.predefined_location_id='.$search_location->id.' OR t.locations_enabled=0)', null, false);
	    	}
	    	
	    	if ($search_by_ids) {
	    		$this->db->where_in('l.id', $search_by_ids);
	    	}
	    	
	    	if ($except_listings) {
	    		$this->db->where_not_in('l.id', $except_listings);
	    	}
	
	    	$query = $this->db->get();
	    	$row = $query->row_array('count_rows');
	    	$this->paginator->setCount($row['count_rows']);
	    	
	    	if ($row['count_rows'] == 0) {
	    		// No listings found
    			return array();
	    	}
    	}
    	// -----------------------------------------------------------------------------------

    	// -----------------------------------------------------------------------------------
    	// Select id of listings that will be shown
    	$this->db->select('l.id');
    	$this->db->from('listings as l');
    	$this->db->join('levels as lev', 'lev.id=l.level_id', 'left');
    	$this->db->join('types as t', 't.id=lev.type_id', 'left');
    	$this->db->join('users as u', 'l.owner_id=u.id', 'left');

    	if (isset($args['what_search']) && $what_listings_ids) {
    		$this->db->where_in('l.id', $what_listings_ids, false);
    	}
    	
    	if (isset($args['where_search']) && $where_listings_ids) {
    		$this->db->where_in('l.id', $where_listings_ids, false);
    	}

    	if (isset($where_radius_ids) && $where_radius_ids) {
    		$this->db->where_in('l.id', $where_radius_ids, false);
    	}
    	
    	if ($only_with_logos) {
    		$this->db->where('lev.logo_enabled', 1);
    		$this->db->not_where('l.logo_file', '');
    	}

    	if (isset($args['search_claimed_listings']) && $args['search_claimed_listings']) {
    		if ($args['search_claimed_listings'] != 'any') {
    			$this->db->join('listings_claims AS lc', 'lc.listing_id=l.id', 'left');
    			if ($args['search_claimed_listings'] == 'ability_to_claim') {
    				$this->db->where('lc.ability_to_claim', 1);
    				$this->db->where('lc.approved', 0);
    			} elseif ($args['search_claimed_listings'] == 'claimed') {
    				$this->db->where('lc.ability_to_claim', 0);
    				$this->db->where('lc.approved', 0);
    				$this->db->where('lc.to_user_id !=', 0);
    			} elseif ($args['search_claimed_listings'] == 'approved_claim') {
    				$this->db->where('lc.to_user_id !=', 0);
    				$this->db->where('lc.approved', 1);
    			}
    		}
    	}
    	
    	if ($current_type) {
    		$this->db->where('t.id', $current_type->id);
    	}
    	if ($page == 'search') {
    		if ($current_type && $current_type->search_type == 'local') {
	    		$this->db->where('t.search_type', 'local');
	    	} elseif ($current_type && $current_type->search_type == 'global') {
	    		$this->db->where('t.search_type', 'global');
	    	}
	    	$this->db->where('t.search_type !=', 'disabled');
    	}
    	
    	if ($search_levels) {
		   	$this->db->where_in('lev.id', $search_levels);
	    }

    	if ($categories_ids) {
    		$this->db->join('listings_in_categories as lic', 'l.id=lic.listing_id', 'left');
    		$this->db->where_in('lic.category_id', $categories_ids);
    		
    		if (is_null($current_type) || $current_type->categories_type == 'global') {
    			$this->db->where('t.categories_type', 'global');
    		}
    	}
    	
    	if ($search_owner_id || $search_owner_login) {
    		if ($search_owner_id)
    			$this->db->like('u.id', $search_owner_id);
    		if ($search_owner_login)
    			$this->db->like('u.login', $search_owner_login);
    	}

    	if (isset($args['search_status']) && $args['search_status']) {
    		$this->db->where('l.status', urldecode($args['search_status']));
    	}
    	
    	if (isset($args['search_users_status']) && $args['search_users_status']) {
    		$this->db->where('u.status', urldecode($args['search_users_status']));
    	}

    	if (isset($args['search_creation_date']) && $args['search_creation_date']) {
    		// Receives date in unix timestamp and also in 'Y-m-d' formats
    		if (($tmstmp = strtotime($args['search_creation_date'])) !== FALSE && strtotime($args['search_creation_date']) != -1)
    			$this->db->where('TO_DAYS(l.creation_date) = ', 'TO_DAYS("' . date("Y-m-d", $tmstmp) . '")', false);
    		else
    			$this->db->where('TO_DAYS(l.creation_date) = ', 'TO_DAYS("' . date("Y-m-d", $args['search_creation_date']) . '")', false);
    	}
    	if (isset($args['search_from_creation_date']) && $args['search_from_creation_date']) {
    		// Receives date in unix timestamp and also in 'Y-m-d' formats
    		if (($tmstmp = strtotime($args['search_from_creation_date'])) !== FALSE && strtotime($args['search_from_creation_date']) != -1)
    			$this->db->where('TO_DAYS(l.creation_date) >= ', 'TO_DAYS("' . date("Y-m-d", $tmstmp) . '")', false);
    		else
    			$this->db->where('TO_DAYS(l.creation_date) >= ', 'TO_DAYS("' . date("Y-m-d", $args['search_from_creation_date']) . '")', false);
    		
    	}
    	if (isset($args['search_to_creation_date']) && $args['search_to_creation_date']) {
    		// Receives date in unix timestamp and also in 'Y-m-d' formats
    		if (($tmstmp = strtotime($args['search_from_creation_date'])) !== FALSE && strtotime($args['search_from_creation_date']) != -1)
    			$this->db->where('TO_DAYS(l.creation_date) <= ', 'TO_DAYS("' . date("Y-m-d", $tmstmp) . '")', false);
    		else
    			$this->db->where('TO_DAYS(l.creation_date) <= ', 'TO_DAYS("' . date("Y-m-d", $args['search_to_creation_date']) . '")', false);
    	}

    	// Get listings ids from results queries of content pages
    	foreach ($search_fields_sql_array AS $search_sql) {
    		if (is_array($search_sql)) {
	    		$condition = key($search_sql);
	    		$sql = $search_sql[$condition];
	    		if ($condition == 'AND')
	    			$this->db->where_in('l.id', $sql, false);
	    		elseif ($condition == 'OR')
	    			$this->db->or_where_in('l.id', $sql, false);
    		}
    	}
    	
    	if ($search_location) {
    		$this->db->join('listings_in_locations AS lil', 'lil.listing_id=l.id', 'left');
    		/*$this->db->like('lil.geocoded_name', $search_location->geocoded_name, 'before');
    		$this->db->where('t.locations_enabled', true);
    		$this->db->where('lev.locations_number > 0');*/
    		$this->db->where('((lil.geocoded_name LIKE "%' . $search_location->geocoded_name . '" AND t.locations_enabled=1 AND lev.locations_number>0) OR lil.predefined_location_id='.$search_location->id.' OR t.locations_enabled=0)', null, false);
    	}
    	
    	if ($search_by_ids) {
    		$this->db->where_in('l.id', $search_by_ids);
    	}
    	
    	if ($except_listings) {
    		$this->db->where_not_in('l.id', $except_listings);
    	}
    	
		$search_sql = str_replace("\n", " ", $this->db->_compile_select());
		$this->db->_reset_select();

    	$this->db->select('l.id');
    	$this->db->from('listings as l');
    	$this->db->join('levels as lev', 'lev.id=l.level_id', 'left');
    	$this->db->join('users as u', 'l.owner_id=u.id', 'left');
    	$this->db->where_in('l.id', $search_sql, false);
    	//$this->db->where_not_in('l.title', 'untranslated'); // except untranslated listings
    	if ($orderby == 'rating') {
    		$this->db->select('AVG(r.value) AS rating');
    		$this->db->join('ratings AS r', 'l.id=r.object_id', 'left');
			$this->db->where('(r.objects_table="listings" OR r.objects_table IS NULL)');
    		$this->db->groupby('l.id');
    	}
    	if ($orderby == 'rev_count') {
    		$this->db->select('COUNT(rev.id) AS rev_count');
    		$this->db->join('reviews AS rev', 'rev.object_id=l.id', 'left');
			$this->db->where('(rev.objects_table="listings" OR rev.objects_table IS NULL)');
			$this->db->where('(rev.status=1 OR rev.status IS NULL)');
    		$this->db->groupby('l.id');
    	}
    	if ($orderby == 'rev_last') {
    		$this->db->select('MAX(rev.date_added) AS rev_last');
    		$this->db->join('reviews AS rev', 'rev.object_id=l.id', 'left');
			$this->db->where('(rev.objects_table="listings" OR rev.objects_table IS NULL)');
			$this->db->where('(rev.status=1 OR rev.status IS NULL)');
    		$this->db->groupby('l.id');
    	}
    	if (($orderby && $direction) && (!isset($what_listings_ids) || !$what_listings_ids) && (!isset($where_listings_ids) || !$where_listings_ids)) {
    		if ($orderby != 'random' && strpos($orderby, 'cf.') === FALSE) {
	    		// Order by date, title, level order, listings ratings, .......
	    		$this->db->order_by($orderby, $direction);
    		}
    	} else {
    		// When any request in 'what' or 'where' search fields,
	    	// then order listings by relevance
				if (isset($what_listings_ids) && $what_listings_ids) {
		    		$orderby = 'CASE l.id';
			    	$i = 1;
			    	foreach ($what_listings_ids AS $id) {
			    		$orderby .= ' WHEN '.$id.' THEN '.$i;
			    		$i++;
			    	}
			    	$orderby .= ' END';
			    	$this->db->order_by($orderby);
		    	}
		    	if (isset($where_listings_ids) && $where_listings_ids) {
		    		$orderby = 'CASE l.id';
			    	$i = 1;
			    	foreach ($where_listings_ids AS $id) {
			    		$orderby .= ' WHEN '.$id.' THEN '.$i;
			    		$i++;
			    	}
			    	$orderby .= ' END';
			    	$this->db->order_by($orderby);
		    	}
    	}
    	// Secondly order by date
    	$this->db->order_by('l.creation_date', 'desc');
    	$query = $this->db->get();
    	$result_array = $query->result_array();
    	$all_ids = array();
    	foreach ($result_array AS $id) {
    		$all_ids[] = $id['id'];
    	}

    	// Order by content field value
    	// select ids, connected with those listings
		if (!empty($all_ids) && strpos($orderby, 'cf.') !== FALSE) {
			$this->db->distinct();
			$this->db->select('l.id');
    		$this->db->from('listings as l');
			$this->db->join('content_fields_groups AS cfg', 'cfg.custom_id=l.level_id', 'left');
			$this->db->where('cfg.custom_name', LISTINGS_LEVEL_GROUP_CUSTOM_NAME);
			
			$this->db->join('content_fields_to_groups AS cftg', 'cftg.group_id=cfg.id', 'left');

			$this->db->join('content_fields_type_'. $content_field_type .'_data AS cftd', 'cftd.object_id=l.id', 'left');
			$this->db->where('cftg.field_id', $content_field_id);
			$this->db->where('(cftd.field_id='.$content_field_id.' OR cftd.field_id IS NULL)');
			$this->db->order_by('cftd.field_value', $direction);
			$this->db->where_in('l.id', $all_ids);
			$query = $this->db->get();
    		$ordered_array = $query->result_array();
    		$ordered_ids = array();
	    	foreach ($ordered_array AS $id) {
	    		$ordered_ids[] = $id['id'];
	    	}
	    	// In result ids first will be ordered ids, than other
	    	$result_ids = array_merge($ordered_ids, array_diff($all_ids, $ordered_ids));
		} else {
			$result_ids = $all_ids;
		}
    	
		if ($orderby == 'random') {
    		shuffle($result_ids);
		}

		if (isset($this->paginator) && $this->paginator) {
			$ids = $this->paginator->getResultIds($result_ids);
		} else {
			$ids = $result_ids;
		}

    	if (!empty($ids)) {
    		// --------------------------------------------------------------------------------------------
    		// Select listings according to found IDs
    		// --------------------------------------------------------------------------------------------
	    	$this->db->select('l.*');
	    	$this->db->from('listings as l');
	    	$_array = array();
	    	$this->db->where_in('l.id', $ids);

	    	// Order by array as is
	    	$orderby = 'CASE l.id';
	    	$i = 1;
	    	foreach ($ids AS $id) {
	    		$orderby .= ' WHEN '.$id.' THEN '.$i;
	    		$i++;
	    	}
	    	$orderby .= ' END';
	    	$this->db->order_by($orderby);
	    	
    		$array = $this->db->get()->result_array();

    		$result_listings = array();
    		foreach ($array AS $row) {
    			$listing_id = $row['id'];

    			$listing = new listing($row['level_id'], $listing_id, $page, $fully_load);
	    		$this->setListingId($listing_id);
	    		$listing->setListingFromArray($row, $this->getListingCategories($listing_id), $this->getListingLocations());
	    		$result_listings[] = $listing;
	    		$this->_cached_listings[$listing_id] = $listing;
    		}
    		return $result_listings;
    		// --------------------------------------------------------------------------------------------
    	} else {
    		// No listings found
    		return array();
    	}
    }
    
    /**
     * Select listings of current user(my listings) from listings table using paginator,
     * this executes optimized method with 3 queries
     *
     * @return array
     */
    public function selectMyListings($orderby = 'id', $direction = 'desc')
    {
    	// Number of rows needs
    	$this->db->select('count(*) as count_rows');
    	$this->db->from('listings as l');
    	$this->db->where('l.owner_id', $this->session->userdata('user_id'));
    	$query = $this->db->get();

    	$count_rows = $query->row_array();
    	$this->paginator->setCount($count_rows['count_rows']);

    	// Select id of listings that will be shown
    	$this->db->select('l.id');
    	$this->db->from('listings as l');
    	$this->db->join('users as u', 'l.owner_id=u.id', 'left');
    	$this->db->where('l.owner_id', $this->session->userdata('user_id'));
    	if ($orderby)
    		$this->db->order_by('l.' . $orderby, $direction);
    	$query = $this->db->get();
    	$ids = $this->paginator->getResultIds($query->result_array());

    	if (!empty($ids)) {
	    	$this->db->select('l.*');
	    	/*$this->db->select('u.login as owner_login');
		    $this->db->select('lev.name as level');
		    $this->db->select('t.name as type');*/
	    	$this->db->from('listings as l');
		    /*$this->db->join('users as u', 'l.owner_id=u.id', 'left');
		    $this->db->join('levels as lev', 'lev.id=l.level_id', 'left');
		    $this->db->join('types as t', 't.id=lev.type_id', 'left');*/
	    	$_array = array();
	    	foreach ($ids AS $id) {
	    		$_array[] = $id['id'];
	    	}
	    	$this->db->where_in('l.id', $_array);
	    	if ($orderby)
    			$this->db->order_by('l.' . $orderby, $direction);
    		$query = $this->db->get();
    		
    		$result_listings = array();
    		foreach ($query->result_array() AS $row) {
    			$listing_id = $row['id'];

	    		$listing = new listing($row['level_id'], $listing_id);
	    		$this->setListingId($listing_id);
	    		$listing->setListingFromArray($row, $this->getListingCategories($listing_id), $this->getListingLocations());
	    		$result_listings[] = $listing;
    		}
    		
    		return $result_listings;
    	} else {
    		// No listings found
    		return array();
    	}
    }
    
    public function getListingById($listing_id = null)
    {
    	if (is_null($listing_id))
    		$listing_id = $this->_listing_id;

    	if (isset($this->_cached_listings[$listing_id]))
    		return $this->_cached_listings[$listing_id];
    	
    	if ($level_id = $this->getLevelIdByListingId($listing_id)) {
    		$this->setListingId($listing_id);
	    	$listing = new listing($level_id, $listing_id);
			$listing->setListingFromArray($this->getListingRowById(), $this->getListingCategories(), $this->getListingLocations());
			$this->_cached_listings[$listing_id] = $listing;
	
			return $this->_cached_listings[$listing_id];
    	} else 
    		return false;
    }
    
    public function getListingRowById($id = null)
    {
    	if (is_null($id)) {
    		$id = $this->_listing_id;
	    }

    	$this->db->select('l.*');
	    $this->db->select('u.login as owner_login');
	    $this->db->select('lev.name as level');
	    $this->db->select('t.name as type');
    	$this->db->from('listings as l');
	    $this->db->join('users as u', 'l.owner_id=u.id', 'left');
	    $this->db->join('levels as lev', 'lev.id=l.level_id', 'left');
	    $this->db->join('types as t', 't.id=lev.type_id', 'left');
    	$this->db->where('l.id', $id);
    	$query = $this->db->get();

    	return $query->row_array();
    }
    
    public function deleteListings($listings_array)
    {
    	if (count($listings_array)) {
	    	foreach ($listings_array AS $id=>$val) {
		    	$this->deleteListingById($id);
	    	}
	    	return true;
    	} else 
    		return false;
    }
    
    public function deleteListingById($listing_id = null)
    {
    	if (empty($listing_id)) {
    		$listing_id = $this->_listing_id;
	    }
	    
	    // Clean cache
		$this->cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG, array('gcategories_count', 'tcategories_count', 'listing_'.$listing_id));
    	
	    $this->db->delete('listings_in_categories', array('listing_id' => $listing_id));
	    $this->db->delete('listings_in_locations', array('listing_id' => $listing_id));
	    $this->db->delete('listings_claims', array('listing_id' => $listing_id));

	    $this->db->delete('images', array('listing_id' => $listing_id));

	    $this->db->delete('files', array('listing_id' => $listing_id));

	    $this->db->delete('videos', array('listing_id' => $listing_id));

	    $this->db->delete('ratings', array('objects_table' => 'listings', 'object_id' => $listing_id));
	    $this->db->delete('reviews', array('objects_table' => 'listings', 'object_id' => $listing_id));
	    
	    // Delete level items from packages
	    $CI = &get_instance();
        if ($CI->load->is_module_loaded("packages")) {
        	$this->db->where('listing_id', $listing_id);
    		$this->db->delete('packages_listings');
        }

	    return $this->db->delete('listings', array('id' => $listing_id));
    }
    
    public function blockListings($listings_ids)
    {
    	// Clean cache
		$this->cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG, array('gcategories_count', 'tcategories_count'));

	    $this->db->set('status', '2');
	    $this->db->where_in('id', $listings_ids);
	    return $this->db->update('listings');
    }
    
    public function activateListings($listings_ids)
    {
    	// Clean cache
		$this->cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG, array('gcategories_count', 'tcategories_count'));

	    $this->db->set('status', '1');
	    $this->db->where_in('id', $listings_ids);
	    return $this->db->update('listings');
    }
    
    /**
     * Select all categories of listing
     *
     * @return array
     */
    public function getListingCategories($listing_id = null)
    {
    	if (!$listing_id)
    		$listing_id = $this->_listing_id;
    	
    	$this->db->select('c.*');
    	$this->db->from('categories AS c');
    	$this->db->join('listings_in_categories AS lic', 'c.id=lic.category_id', 'left');
    	$this->db->where('lic.listing_id', $listing_id);
    	$this->db->order_by('lic.id');
    	$query = $this->db->get();

    	$categories_db = $query->result_array();
    	
    	$categories = array();
    	foreach ($categories_db AS $category_row) {
    		$category = new category;
    		$category->setCategoryFromArray($category_row);
    		$categories[] = $category;
    	}
    	
    	return $categories;
    }
    
    /**
     * Select all locations of listing
     *
     * @return array
     */
    public function getListingLocations($listing_id = null)
    {
    	if (!$listing_id)
    		$listing_id = $this->_listing_id;

    	$this->db->select('lic.*');
    	$this->db->from('listings_in_locations AS lic');
    	$this->db->where('lic.listing_id', $listing_id);
    	$this->db->order_by('lic.id');
    	$query = $this->db->get();

    	$locations_db = $query->result_array();
    	
    	$locations = array();
	    foreach ($locations_db AS $location_row) {
	    	$location = new listingLocation;
	    	$location->setLocationFromArray($location_row);
	    	$locations[] = $location;
	    }
    	return $locations;
    }

    public function prolongListing($listing)
    {
    	if ($listing->type->categories_type == 'global')
			$this->cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG, array('categories_count-0'));
		else 
			$this->cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG, array('categories_count-' . $listing->type->id));

    	$this->db->set('status', 1); // active
    	$this->db->set('expiration_date', date("Y-m-d H:i:s", (mktime() + (
    											($listing->level->active_days) +
    											($listing->level->active_months*30) +
    											($listing->level->active_years*365)
    											)*86400)));
    	$this->db->set('last_modified_date', date("Y-m-d H:i:s"));
    	$this->db->set('was_prolonged_times', 'was_prolonged_times+1', false);
    	$this->db->where('id', $this->_listing_id);
    	return $this->db->update('listings');
    }
    
    public function getMyListingsCount()
    {
    	$this->db->select('count(*) AS listings_count');
    	$this->db->from('listings');
    	$this->db->where('owner_id', $this->session->userdata('user_id'));
    	$query = $this->db->get();
    	$row = $query->row_array();
    	
    	return $row['listings_count'];
    }
    
    public function getListingArrayById($listing_id)
    {
    	$query = new query;
    	$query->select();
    	$query->from('listings');
    	$query->where('id', $listing_id);
    	
    	$this->db->execQuery($query);
    	
    	return $this->db->rowAsArray();
    }
    
    public function suspendExpiredActiveListings()
    {
    	$this->db->select('l.id');
    	$this->db->select('l.title');
    	$this->db->select('u.login AS owner_login');
    	$this->db->select('u.email AS owner_email');
    	$this->db->from('listings AS l');
    	$this->db->join('levels AS lev', 'l.level_id=lev.id', 'left');
    	$this->db->join('users AS u', 'u.id=l.owner_id', 'left');
    	$this->db->where('l.expiration_date <=', 'NOW()', false);
    	$this->db->where('l.status', 1);
    	$this->db->where('u.status', 2);
    	$this->db->where('(lev.active_years!=0 OR lev.active_months!=0 OR lev.active_days!=0)', null, false);
    	$query = $this->db->get();

    	$listings = $query->result_array();

    	$ids = array();
    	foreach ($listings AS $listing) {
    		$ids[] = $listing['id'];

    		$event_params = array(
				'LISTING_ID' => $listing['id'], 
				'LISTING_TITLE' => $listing['title'], 
				'RECIPIENT_NAME' => $listing['owner_login'],
				'RECIPIENT_EMAIL' => $listing['owner_email']
			);
    		$notification = new notificationSender('Listing expiration');
			$notification->send($event_params);
    		events::callEvent('Listing expiration', $event_params);
    	}
    	
    	if (!empty($ids)) {
	    	$this->db->set('status', 3);
	    	$this->db->where_in('id', $ids);
	    	return $this->db->update('listings');
    	}
    }
    
    // --------------------------------------------------------------------------------------------
    // Claiming methods
    // --------------------------------------------------------------------------------------------
    public function getClaimRow($listing_id)
    {
    	$this->db->select();
    	$this->db->from('listings_claims');
    	$this->db->where('listing_id', $listing_id);
    	return $this->db->get()->row_array();
    }
    
    public function setClaim($claim_row_id)
    {
    	$this->db->set('ability_to_claim', 0);
    	$this->db->set('to_user_id', $this->session->userdata('user_id'));
    	$this->db->where('id', $claim_row_id);
    	return $this->db->update('listings_claims');
    }
    
    public function approveClaim($listing_id)
    {
    	$this->db->set('approved', 1);
    	$this->db->set('ability_to_claim', 0);
    	$this->db->where('listing_id', $listing_id);
    	return $this->db->update('listings_claims');
    }
    
    public function declineClaim($listing_id)
    {
    	$this->db->set('approved', 0);
    	$this->db->set('ability_to_claim', 1);
    	$this->db->set('to_user_id', 0);
    	$this->db->where('listing_id', $listing_id);
    	return $this->db->update('listings_claims');
    }
    
    public function delegateListingToUser($listing_id, $user_id)
    {
    	$this->db->set('owner_id', $user_id);
    	$this->db->where('id', $listing_id);
    	return $this->db->update('listings');
    }
    
    public function rollBackListingFromUser($listing_id, $initial_user_id)
    {
    	$this->db->set('owner_id', $initial_user_id);
    	$this->db->where('id', $listing_id);
    	return $this->db->update('listings');
    }
    // --------------------------------------------------------------------------------------------
}
?>