<?php
include_once(MODULES_PATH . 'locations_predefined/classes/locations_level.class.php');
include_once(MODULES_PATH . 'locations_predefined/classes/predefined_location.class.php');
include_once(MODULES_PATH . 'google_maps/classes/location_geoname.class.php');

class locationsModel extends model
{
    private $locations_db;
    private $_loc_level_id;
    private $_location_id;
    
    // --------------------------------------------------------------------------------------------
    // locations levels methods
    // --------------------------------------------------------------------------------------------
    public function setLocationsLevelId($level_id)
    {
    	$this->_loc_level_id = $level_id;
    }

    public function selectAllLevels()
    {
        $this->db->select('id');
        $this->db->select('order_num');
        $this->db->select('name');
        $this->db->from('locations_levels');
        $this->db->order_by('order_num');
		$result = $this->db->get()->result_array();
		
		$locations_levels = array();
		foreach ($result AS $row) {
			$locations_level = new locations_level;
			$locations_level->setLocationsLevelFromArray($row);
			$locations_levels[] = $locations_level;
		}
		return $locations_levels;
    }
    
    public function getLocationsLevelsCount()
    {
    	$this->db->select('count(*) AS levels_count');
        $this->db->from('locations_levels');
		$query = $this->db->get();
		$row = $query->row_array();

        return $row['levels_count'];
    }
    
    /**
     * saves the order of locations levels by its weight
     *
     * @param string $serialized_order
     */
    public function setLocLevelsOrder($serialized_order)
    {
    	$a = explode("=", $serialized_order);
    	$start = 1;
    	foreach ($a AS $row) {
    		$b = explode("&", $row);
    		foreach ($b AS $id) {
    			$id = trim($id, "_id");
    			if (is_numeric($id)) {
    				$this->db->set('order_num', $start++);
    				$this->db->where('id', $id);
    				$this->db->update('locations_levels');
    			}
    		}
    	}
    }
    
    /**
     * is there locations level with such name in the DB?
     *
     * @param string $name
     */
    public function is_loc_level_name($name)
    {
    	$this->db->select();
		$this->db->from('locations_levels');
		$this->db->where('name', $name);
		if (!is_null($this->_loc_level_id)) {
			$this->db->where('id !=', $this->_loc_level_id);
		}
		$query = $this->db->get();

		return $query->num_rows();
    }
    
    public function getNewLocationsLevel()
    {
    	$locations_level = new locations_level;
    	return $locations_level;
    }
    
    public function saveLocationsLevel($form_result)
    {
    	$this->db->select_max('order_num');
    	$query = $this->db->get('locations_levels');
    	if ($row = $query->row())
    		$order_num = $row->order_num + 1;
    	else 
    		$order_num = 1;
    	
        $this->db->set('name', $form_result['name']);
        $this->db->set('order_num', $order_num);
        if ($this->db->insert('locations_levels')) {
        	$system_settings = registry::get('system_settings');
        	if (@$system_settings['language_areas_enabled']) {
        		translations::saveTranslations(array('locations_levels', 'name', $this->db->insert_id()));
        	}
        	return true;
        }
    }
    
    public function getLocationsLevelFromForm($form_result)
    {
		$locations_level = new locations_level;
		$locations_level->setLocationsLevelFromArray($form_result);

        return $locations_level;
    }
    
    public function getLocationsLevelById()
    {
    	$this->db->select();
        $this->db->where('id', $this->_loc_level_id);
        $query = $this->db->get('locations_levels');

        if ($query->num_rows()) {
            $locations_level = new locations_level;
            $locations_level->setLocationsLevelFromArray($query->row_array());
            return $locations_level;
        } else {
            return false;
        }
    }
    
    public function saveLocationsLevelById($form_result)
    {
    	$this->db->set('name', $form_result['name']);
        $this->db->where('id', $this->_loc_level_id);
        return $this->db->update('locations_levels');
    }
    
    public function deleteLocationsLevelById()
    {
    	$this->db->where('id', $this->_loc_level_id);
        $this->db->delete('locations_levels');

    	// Update all levels weights after delete
    	$loc_levels = $this->selectAllLevels();
    	$count = 1;
    	foreach ($loc_levels AS $level) {
    		$this->db->set('order_num', $count);
    		$this->db->where('id', $level->id);
    		$this->db->update('locations_levels');
    		$count++;
    	}

    	// Delete unnecessary locations after appropriate levels were deleted
        $locations_db = $this->selectLocationsFromDB();

        $level_max = count($loc_levels);
        
        $locations_to_delete = array();
        $curr_level = 1;
        for ($i = 0; $i < count($locations_db); $i++) {
            if ($locations_db[$i]['parent_id'] == 0) {
                if ($curr_level > $level_max) {
                    $locations_to_delete[] = $locations_db[$i]['id'];
                }
                $this->recursiveLocationsToDeleteFind($locations_db, $i, $curr_level, $level_max, $locations_to_delete);
            }
        }

        if (!empty($locations_to_delete)) {
            $this->db->where_in('id', $locations_to_delete);
            $this->db->delete('locations');
        }
        return true;
    }
    
    public function recursiveLocationsToDeleteFind($locations_db, $i, &$curr_level, $level_max, &$locations_to_delete)
    {
        $this_level = $curr_level;
        $this_level++;
        for ($j = 0; $j < count($locations_db); $j++) {
            if ($locations_db[$j]['parent_id'] == $locations_db[$i]['id']) {
                if ($this_level > $level_max) {
                    $locations_to_delete[] = $locations_db[$j]['id'];
                }
                $this->recursiveLocationsToDeleteFind($locations_db, $j, $this_level, $level_max, $locations_to_delete);
            }
        }
    }
    
    // --------------------------------------------------------------------------------------------
    // Admin Locations functions
    // --------------------------------------------------------------------------------------------
    public function setLocationId($location_id)
    {
    	$this->_location_id = $location_id;
    }

    public function selectLocationsFromDB()
    {
    	if (!empty($this->locations_db)) {
    		return $this->locations_db;
    	}

    	// Select each column separately because there may be error when language areas enabled,
    	// order by statement doesn't auto-rewrite when we select from another language
        $this->db->select('id');
        $this->db->select('parent_id');
        $this->db->select('tree_path');
        $this->db->select('use_as_label');
        $this->db->select('name');
        $this->db->select('seo_name');
        $this->db->select('geocoded_name');
		$this->db->from('locations');
        $this->db->order_by('name');
        $query = $this->db->get();
        
        $this->locations_db = $query->result_array();

        return $query->result_array();
    }

    public function buildLocationsDomElements($parent_id = 0)
    {
    	$locations_db = $this->selectLocationsFromDB();
    	
    	$levels_count = $this->getLocationsLevelsCount();
    	
    	if ($parent_id != 0)
    		$locations_chain_count = count($this->getLocationsChainFromId($parent_id));
    	else 
    		$locations_chain_count = 1;
    	$curr_levels_count = $locations_chain_count + 1;

    	$json = array();
        for ($i=0; $i<count($locations_db); $i++) {
            if ($locations_db[$i]['parent_id'] == $parent_id) {
            	$state = "";
				for ($j=0; $j<count($locations_db); $j++) {
					if ($locations_db[$j]['parent_id'] == $locations_db[$i]['id']) {
						$state = "closed";
					}
				}
				
				if ($locations_db[$i]['use_as_label'])
					$data = '<b>' . _utf8_encode($locations_db[$i]['name']) . '</b>&nbsp;&nbsp;&nbsp;(' . $locations_db[$i]['seo_name'] . ')';
				else
					$data = _utf8_encode($locations_db[$i]['name']) . '&nbsp;&nbsp;&nbsp;(' . $locations_db[$i]['seo_name'] . ')';
				// Insert 'create child' option only if it is not last level
				if ($curr_levels_count < $levels_count)
					$data .= '&nbsp;&nbsp;&nbsp;<a href="' . site_url('admin/locations/create_child/' . $locations_db[$i]['id'] . '/') . '" class="tree_option" onClick="document.location.href=this.href">' . _utf8_encode(LANG_LOCATION_CREATE_CHILD) . '</a>';
				$data .= '&nbsp;&nbsp;&nbsp;<a href="' . site_url('admin/locations/edit/' . $locations_db[$i]['id'] . '/') . '" class="tree_option" onClick="document.location.href=this.href">' . _utf8_encode(LANG_LOCATION_EDIT) . '</a>';
				$data .= '&nbsp;&nbsp;&nbsp;<a href="' . site_url('admin/locations/delete/' . $locations_db[$i]['id'] . '/') . '" class="tree_option" onClick="document.location.href=this.href">' . _utf8_encode(LANG_LOCATION_DELETE) . '</a>';
				$data .= '&nbsp;&nbsp;&nbsp;<a href="' . site_url('admin/locations/label/' . $locations_db[$i]['id'] . '/') . '" class="tree_option" onClick="document.location.href=this.href">' . _utf8_encode(LANG_LOCATION_LABEL) . '</a>';
				
				$json[] = array(
								"data" => $data,
								"attributes" => array("id" => $locations_db[$i]['id']),
								"state" => $state,
						);
            }
        }

        return json_encode($json);
    }
    
    public function saveLocations($list)
    {
        $values = array();
        $a = explode('*', $list);
        foreach ($a as $v) {
            if (!empty($v)) {
                $item = str_replace(']', '', str_replace('[', '', $v));
                $b = explode('-', $item);
                if (count($b) == 1) {
                    $id = $b[0];
                    $parent_id = 0;
                } else {
                    $id = $b[1];
                    $parent_id = $b[0];
                }

                $values[$id] = $parent_id;
            }
        }
        if (count($values)) {
        	foreach ($values AS $id=>$parent_id) {
        		$this->db->set('parent_id', $parent_id);
        		$this->db->where('id', $id);
        		$this->db->update('locations');
        	}
        	$locations_db = $this->selectLocationsFromDB();
        	foreach ($values AS $id=>$parent_id) {
        		$this->db->select('id');
        		$this->db->select('parent_id');
        		$this->db->from('locations');
        		$this->db->like('tree_path', '-'.$id, 'both');
        		$result = $this->db->get()->result_array();
        		
        		foreach ($result AS $location_row) {
        			$path = array();
					$path[] = $location_row['id'];
					$this->getLocationsTreePathRecursively($location_row['parent_id'], $locations_db, $path);
					$path[] = 0;
					$path = array_reverse($path);
					$path_string = '-'.implode('-', $path);

					// --------------------------------------------------------------------------------------------
					// Geocode and resave listings_in_locations
					// --------------------------------------------------------------------------------------------
					$location = $this->getLocationById($location_row['id']);
			    	$geocoder = new locationGeoname;
			    	if ($geocoded_name = $geocoder->geonames_request($location->getChainAsString())) {
			    		$this->db->set('geocoded_name', $geocoded_name);
			    	}
			    	$this->db->set('tree_path', $path_string);
					$this->db->where('id', $location_row['id']);
					$this->db->update('locations');
					
					if ($geocoded_name) {
						$this->db->set('geocoded_name', $geocoded_name);
				    	$this->db->where('predefined_location_id', $location_row['id']);
				    	$this->db->update('listings_in_locations');
					}
					// --------------------------------------------------------------------------------------------
        		}
        	}
        }
        return true;
    }
    
    public function deleteLocations($list)
    {
        $a = explode('*', $list);
        foreach ($a as $id) {
            if (!empty($id)) {
                $this->deleteLocationById($id);
            }
        }
        return true;
    }
    
    public function labelLocations($list)
    {
		$a = array_unique(explode('*', $list));
		foreach ($a as $id) {
			if (!empty($id)) {
				$this->labelLocationById($id);
			}
		}
		return true;
    }

    public function getNewLocation()
    {
    	$location = new predefinedLocation;
    	return $location;
    }
    
    public function getLocationFromForm($form)
    {
    	$location = new predefinedLocation;
    	$location->setLocationFromArray($form);
    	return $location;
    }
    
    public function is_location_seoname($seo_name)
    {
    	$this->db->select();
		$this->db->from('locations');
		$this->db->where('seo_name', $seo_name);
		if (!is_null($this->_location_id)) {
			$this->db->where('id !=', $this->_location_id);
		}
		$query = $this->db->get();

		return $query->num_rows();
    }

    public function saveLocation($form, $parent_id = 0)
    {
    	$this->db->set('parent_id', $parent_id);
    	$this->db->set('name', $form['name']);
    	$this->db->set('seo_name', $form['seo_name']);
    	if ($this->db->insert('locations')) {
    		$new_id = $this->db->insert_id();

    		$this->db->select('id');
    		$this->db->select('name');
	        $this->db->select('parent_id');
			$this->db->from('locations');
	        $locations_db = $this->db->get()->result_array();

	        $path = array();
			$path[] = $new_id;
			$this->getLocationsTreePathRecursively($parent_id, $locations_db, $path);
			$path[] = 0;
			$path = array_reverse($path);
			$path_string = '-'.implode('-', $path);

			// --------------------------------------------------------------------------------------------
			// Geocode and resave listings_in_locations
			// --------------------------------------------------------------------------------------------
			$locations = array();
			foreach ($path AS $id) {
				foreach ($locations_db AS $location_row) {
					if ($location_row['id'] == $id) {
						$locations[] = $location_row['name'];
					}
				}
			}
	    	$geocoder = new locationGeoname;
	    	if ($geocoded_name = $geocoder->geonames_request(implode(', ', $locations))) {
	    		$this->db->set('geocoded_name', $geocoded_name);
	    	}
	    	$this->db->set('tree_path', $path_string);
			$this->db->where('id', $new_id);
			$this->db->update('locations');
			
			if ($geocoded_name) {
				$this->db->set('geocoded_name', $geocoded_name);
		    	$this->db->where('predefined_location_id', $new_id);
		    	$this->db->update('listings_in_locations');
			}
			// --------------------------------------------------------------------------------------------

        	$system_settings = registry::get('system_settings');
        	if (@$system_settings['language_areas_enabled']) {
        		translations::saveTranslations(array('locations', 'name', $new_id));
        	}
        	return $new_id;
    	}
    }
    
    public function getLocationsTreePathRecursively($parent_id, $locations_array, &$path)
	{
		foreach ($locations_array AS $location_row) {
			if ($location_row['id'] == $parent_id) {
				$path[] = $location_row['id'];
				$this->getLocationsTreePathRecursively($location_row['parent_id'], $locations_array, $path);
				break;
			}
		}
	}
    
    public function getLocationById($location_id = null)
    {
    	if (is_null($location_id)) {
    		$location_id = $this->_location_id;
    	}
    	
    	$this->db->select();
    	$this->db->from('locations');
    	$this->db->where('id', $location_id);
    	$query = $this->db->get();

    	if ($query->num_rows()) {
	    	$location = new predefinedLocation;
	    	$location->setLocationFromArray($query->row_array());
	    	return $location;
    	}
    }
    
    public function saveLocationById($form)
    {
    	$this->db->set('name', $form['name']);
    	$this->db->set('seo_name', $form['seo_name']);
    	$this->db->where('id', $this->_location_id);
    	if ($this->db->update('locations')) {
    		// --------------------------------------------------------------------------------------------
			// Geocode and resave listings_in_locations
			// --------------------------------------------------------------------------------------------
			$location = $this->getLocationById($this->_location_id);
	    	$geocoder = new locationGeoname;
	    	if ($geocoded_name = $geocoder->geonames_request($location->getChainAsString())) {
	    		$this->db->set('geocoded_name', $geocoded_name);
		    	$this->db->where('predefined_location_id', $this->_location_id);
		    	$this->db->update('listings_in_locations');
	    	
		    	$this->db->set('geocoded_name', $geocoded_name);
		    	$this->db->where('id', $this->_location_id);
		    	$this->db->update('locations');
		    }
	    	// --------------------------------------------------------------------------------------------
	    	return true;
    	}
    }
    
    public function deleteLocationById($location_id = null)
    {
    	if (is_null($location_id)) {
    		$location_id = $this->_location_id;
    	}
    	
    	$locations_db = $this->selectLocationsFromDB();
    	
    	foreach ($locations_db AS $location_row) {
    		if ($location_row['parent_id'] == $location_id) {
    			$this->deleteLocationById($location_row['id']);
    		}
    	}
    	
    	if ($location = $this->getLocationById($location_id)) {
    		$this->db->set('predefined_location_id', 0);
	    	$this->db->set('location', $location->geocoded_name);
	    	$this->db->set('use_predefined_locations', false);
	    	$this->db->where('predefined_location_id', $location_id);
	    	$this->db->where('use_predefined_locations', true);
	    	$this->db->update('listings_in_locations');
    	}
    	
    	$this->db->where('id', $location_id);
    	return $this->db->delete('locations');
    }
    
    public function labelLocationById($location_id = null)
    {
    	if (is_null($location_id)) {
    		$location_id = $this->_location_id;
    	}
    	
    	$this->db->select('use_as_label');
    	$this->db->where('id', $location_id);
    	$row = $this->db->get('locations')->row_array();

    	if ($row['use_as_label'])
    		$this->db->set('use_as_label', false);
    	else
    		$this->db->set('use_as_label', true);
    	$this->db->where('id', $location_id);
    	return $this->db->update('locations');
    }
    
    public function getLabeledLocations()
    {
    	$this->db->select('id');
        $this->db->select('parent_id');
        $this->db->select('tree_path');
        $this->db->select('use_as_label');
        $this->db->select('name');
        $this->db->select('seo_name');
        $this->db->select('geocoded_name');
		$this->db->from('locations');
        $this->db->where('use_as_label', true);
        $this->db->order_by('name');
        $result = $this->db->get()->result_array();
        
        $locations = array();
        foreach ($result AS $row) {
        	$location = new predefinedLocation;
        	$location->setLocationFromArray($row);
        	$locations[] = $location;
        }

        return $locations;
    }
    
    public function getLocationBySeoName($seo_name)
    {
    	$this->db->select();
		$this->db->from('locations');
        $this->db->where('seo_name', $seo_name);
        $query = $this->db->get();
        
        if ($query->num_rows()) {
        	$row = $query->row_array();
        	$location = new predefinedLocation;
        	$location->setLocationFromArray($row);
        	return $location;
        } else 
        	return false;
    }

    public function getLocationsChainFromId($parent_id)
    {
    	// Explode path and remove root level
    	$parent_location_obj = $this->getLocationById($parent_id);
    	$parent_locations = array_reverse(explode('-', $parent_location_obj->tree_path));
    	

    	$selected_locations_chain = array();
		foreach ($parent_locations as $location_id) {
			if ($location_id != 0)
				$selected_locations_chain[] = $this->getLocationById($location_id);
		}
		return $selected_locations_chain;
    }

    // Select locations that are on the same level as $parent_id
    public function selectAllSisters($parent_id, $location_id = null)
    {
    	$locations_db = $this->selectLocationsFromDB();
    	
    	$locations_array = array();
    	foreach ($locations_db as $location) {
			if ($parent_id == $location['parent_id']) {
				$predefined_location = new predefinedLocation;
				$predefined_location->setLocationFromArray($location);
				if ($location_id == $predefined_location->id) {
					$predefined_location->setSelected();
				}
				$locations_array[] = $predefined_location;
			}
		}
		return $locations_array;
    }
    

    public function synchronize_locations()
    {
    	$locations_levels_count = $this->getLocationsLevelsCount();
    	
    	$CI = &get_instance();
    	if ($CI->load->is_module_loaded('i18n')) {
    		$CI->load->model('languages', 'i18n');
			$languages = $CI->languages->getLanguages();
    	} else {
    		$languages[] = array('code' => 'en');
    	}
    	$default_language_code = $CI->config->item('default_language');
    	
    	$this->db->distinct();
    	$this->db->select('geocoded_name');
    	$this->db->from('listings_in_locations');
    	$this->db->where('geocoded_name !=', '');
    	$array = $this->db->get()->result_array();
    	$tree_array = array();
    	foreach ($array AS $row) {
    		$row_array = explode(', ', $row['geocoded_name']);
    		$this->buildLocationsTreeRecursively(0, array_reverse($row_array), $tree_array);
    	}
    	$this->saveLocationsTreeRecursively($locations_levels_count, 0, $languages, $default_language_code, $tree_array);
    	return true;
    }
    
    private function buildLocationsTreeRecursively($i, $row_array, &$tree)
    {
    	if (isset($row_array[$i]) && !isset($tree[$row_array[$i]])) {
    		$tree[$row_array[$i]] = null;
    	}
    	if ($i<count($row_array)) {
    		$this->buildLocationsTreeRecursively($i+1, $row_array, $tree[$row_array[$i]]);
    	}
    }
    
    public function saveLocationsTreeRecursively($locations_levels_count, $parent_location_id, $languages, $default_language_code, $tree, &$orig_row_array = array())
    {
    	if (is_array($tree)) {
    		$CI = &get_instance();

	    	foreach ($tree AS $name=>$thread) {
	    		$row_array = $orig_row_array;
	    		$row_array[] = $name;

	    		if (count($row_array) >= $locations_levels_count && is_array($thread)) {
	    			$this->saveLocationsTreeRecursively($locations_levels_count, $parent_location_id, $languages, $default_language_code, $thread, $row_array);
	    			return ;
	    		} elseif (is_null($thread)) {
	    			$geocoded_name_array = array_slice($row_array, 0, $locations_levels_count-1);
	    			$geocoded_name_array[] = end($row_array);
	    		} else {
	    			$geocoded_name_array = $row_array;
	    		}

	    		$translation_array = array();
	    		$last_name = $geocoded_name_array;
				$en_lang_name = end($last_name);
				foreach ($languages AS $lang) {
					if ($CI->load->is_module_loaded('i18n') && $lang['db_code'] != 'en')
						$translation_array['locations']['name'][0][$lang['db_code']] = $CI->languages->translateString($en_lang_name, 'en', $lang['code']);

					if ($lang['code'] == $default_language_code) {
						if ($CI->load->is_module_loaded('i18n') && $lang['db_code'] != 'en')
							$form['name'] = $translation_array['locations']['name'][0][$default_language_code];
						else
							$form['name'] = $en_lang_name;
					}
				}
				$form['seo_name'] = friendly_seo_string($en_lang_name);
				
				if ($CI->load->is_module_loaded('i18n'))
					$CI->session->set_userdata('translation_array', base64_encode(serialize($translation_array)));

				if (!($existed_location_id = $this->isLocation($geocoded_name_array))) {
					if ($this->is_location_seoname($form['seo_name']))
						$form['seo_name'] = $form['seo_name'] . '-1';
					$new_parent_location_id = $this->saveLocation($form, $parent_location_id);
					if (is_array($thread)) {
		    			$this->saveLocationsTreeRecursively($locations_levels_count, $new_parent_location_id, $languages, $default_language_code, $thread, $row_array);
		    		}
		    		$existed_location_id = $new_parent_location_id;
				}
				if (is_null($thread)) {
		    		$this->synchronizeLicAndLocation(implode(', ', array_reverse($row_array)), $existed_location_id);
				}
	    	}
    	}
    }
    
    public function isLocation($name_array)
    {
    	foreach ($name_array AS $name) {
    		$this->db->select('id');
    		$this->db->from('locations');
    		$this->db->where('name', $name);
    		$query = $this->db->get();
    		if ($query->num_rows())
    			$row = $query->row_array();
    		else 
    			return false;
    		$result[] = $row['id'];
    	}
    	$this->db->select('id');
    	$this->db->from('locations');
    	$this->db->where('tree_path', '-0-' . implode('-', $result));
    	return $this->db->get()->num_rows();
    }
    
    public function synchronizeLicAndLocation($lic_geocoded_name, $location_id)
    {
    	// --------------------------------------------------------------------------------------------
		// Geocode location
		// --------------------------------------------------------------------------------------------
    	$geocoder = new locationGeoname;
    	if ($geocoded_name = $geocoder->geonames_request($lic_geocoded_name)) {
    		$this->db->set('geocoded_name', $geocoded_name);
    	}
    	// --------------------------------------------------------------------------------------------
    	
    	$this->db->set('predefined_location_id', $location_id);
    	$this->db->where('geocoded_name', $lic_geocoded_name);
    	return $this->db->update('listings_in_locations');
    }
}
?>