<?php
include_once(MODULES_PATH . 'ratings_reviews/classes/review.class.php');

class reviewsModel extends Model
{
	protected $mode = 'admin';
	
	public function setMode($mode)
	{
		$this->mode = $mode;
	}
	
	public function getReviewsArray($objects_table, $object_id)
	{
		$this->db->select('rev.*');
		$this->db->from('reviews AS rev');
		$this->db->where('rev.objects_table', $objects_table);
		$this->db->where('rev.object_id', $object_id);
		$this->db->order_by('rev.date_added');
		$query = $this->db->get();

		return $query->result_array();
	}
	
	public function getReviewsStructured($objects_table, $object_id)
	{
		$reviews_array = $this->getReviewsArray($objects_table, $object_id);
		
		$reviews_structured_array = array();
		$this->buildReviewsRecursively($objects_table, $object_id, $reviews_structured_array, $reviews_array);
		
		return $reviews_structured_array;
	}
	
	private function buildReviewsRecursively($objects_table, $object_id, &$reviews_structured_array, $reviews_db, $parent_review = null)
	{
		if (is_null($parent_review))
			$parent_review_id = 0;
		else
			$parent_review_id = $parent_review->id;
		
		foreach ($reviews_db AS $key=>$reviews_item) {
			if ($reviews_item['parent_id'] == $parent_review_id) {
				$review = new review($reviews_item);
				$review->setMode($this->mode);
				$review->setRating($objects_table, $object_id);
				if (!is_null($parent_review)) {
					$parent_review->attachChild($review);
					unset($reviews_db[$key]);
				} else {
					$reviews_structured_array[] = $review;
				}
				$this->buildReviewsRecursively($objects_table, $object_id, $reviews_structured_array, $reviews_db, $review);
			}
		}
	}

	public function getReviewsCount($objects_table, $object_id)
	{
		$this->db->select('COUNT(id) AS reviews_count');
		$this->db->from('reviews');
		$this->db->where('objects_table', $objects_table);
		$this->db->where('object_id', $object_id);
		/*if ($this->mode != 'admin') {
			$this->db->where('status', 1);
		}*/
		$row = $this->db->get()->row_array();
		return $row['reviews_count'];
	}

	public function addReview($objects_table, $object_id, $review, $parent_id, $anonym_name, $anonym_email)
	{
		if ($this->session->userdata('user_id')) {
			$user_id = $this->session->userdata('user_id');
		} else {
			$user_id = 0;
		}
		
		$ip = $this->input->ip_address();

		$event_params = array(
			'OBJECTS_TABLE' => $objects_table,
			'OBJECT_ID' => $object_id,
			'REVIEW_BODY' => $review,
			'PARENT_REIVEW_ID' => $parent_id,
			'USER_ID' => $user_id,
			'ANONYM_NAME' => $anonym_name,
			'ANONYM_EMAIL' => $anonym_email,
			'IP' => $ip,
		);
		events::callEvent('Review creation', $event_params);
		$this->cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG, array('reviews_' . $objects_table . '_' . $object_id));

		$this->db->set('objects_table', $objects_table);
		$this->db->set('object_id', $object_id);
		$this->db->set('review', $review);
		$this->db->set('parent_id', $parent_id);
		$this->db->set('anonym_name', $anonym_name);
		$this->db->set('anonym_email', $anonym_email);
		$this->db->set('user_id', $user_id);
		$this->db->set('status', 1);
		$this->db->set('ip', $ip);
		$this->db->set('date_added', date("Y-m-d H:i:s"));
		return $this->db->insert('reviews');
	}
	
	public function getReviewById($review_id)
	{
		$this->db->select('rev.*');
		$this->db->from('reviews AS rev');
		$this->db->where('rev.id', $review_id);
		$query = $this->db->get();

		$review = new review($query->row_array());
		return $review;
	}
	
	public function saveReviewById($review_id, $review_body)
	{
		$this->db->set('review', $review_body);
		$this->db->where('id', $review_id);
		return $this->db->update('reviews');
	}
	
	public function deleteReviews($reviews_db, $reviews_array)
    {
    	if (count($reviews_array)) {
    		$reviews_to_delete = array();
    		foreach ($reviews_array AS $review_item=>$val) {
    			$reviews_to_delete[] = $review_item;
	    		foreach ($reviews_db AS $review_item_db) {
	    			if ($review_item == $review_item_db['parent_id']) {
	    				$reviews_to_delete[] = $review_item_db['id'];
	    				$this->findReviewsToDeleteRecursively($reviews_db, $review_item_db['id'], $reviews_to_delete);
	    			}
	    		}
    		}
    		
    		foreach ($reviews_to_delete AS $review_id) {
    			$review = $this->getReviewById($review_id);
    			$review->object->cleanCache();
    		}

    		$this->db->where_in('id', $reviews_to_delete);
    		return $this->db->delete('reviews');
    	}
    }
    
    private function findReviewsToDeleteRecursively($reviews_db, $parent_id, &$reviews_to_delete)
    {
    	foreach ($reviews_db AS $review_item_db) {
	    	if ($parent_id == $review_item_db['parent_id']) {
	    		$reviews_to_delete[] = $review_item_db['id'];
	    		$this->findReviewsToDeleteRecursively($reviews_db, $review_item_db['id'], $reviews_to_delete);
	    	}
	    }
    }
    
    public function spamReviews($reviews_ids)
    {
    	if (count($reviews_ids)) {
    		foreach ($reviews_ids AS $reviews_id) {
    			$review = $this->getReviewById($reviews_id);
    			$review->object->cleanCache();
    		}
    		
    		$this->db->set('status', 2);
    		$this->db->where_in('id', $reviews_ids);
    		return $this->db->update('reviews');
    	}
    }
    
    public function activateReviews($reviews_ids)
    {
    	if (count($reviews_ids)) {
    		foreach ($reviews_ids AS $reviews_id) {
    			$review = $this->getReviewById($reviews_id);
    			$review->object->cleanCache();
    		}

    		$this->db->set('status', 1);
    		$this->db->where_in('id', $reviews_ids);
    		return $this->db->update('reviews');
    	}
    }
    
    /**
     * Select all reviews table using paginator,
     * this executes optimized method with 3 queries
     *
     * @return array
     */
    public function selectReviews($objects_table, $orderby = 'id', $direction = 'asc', $args = array())
    {
    	// -----------------------------------------------------------------------------------
    	// Number of rows needs
    	//
    	$this->db->select('count(*) as count_rows');
    	$this->db->from('reviews AS r');
    	$this->db->where('r.objects_table', $objects_table);
    	if (isset($args['search_login'])) {
    		$this->db->like('u.login', urldecode(html_entity_decode($args['search_login'])));
    	}
    	if (isset($args['search_anonyms'])) {
    		if ($args['search_anonyms'] == 'anonyms') {
    			$this->db->where('r.user_id', 0);
    		}
    	}
    	if (isset($args['search_status'])) {
    		$this->db->where('r.status', urldecode($args['search_status']));
    	}
    	if (isset($args['search_date_added'])) {
    		$this->db->where('TO_DAYS(r.date_added) = ', 'TO_DAYS("' . date("Y-m-d", $args['search_date_added']) . '")', false);
    	}
    	if (isset($args['search_from_date_added'])) {
    		$this->db->where('TO_DAYS(r.date_added) >= ', 'TO_DAYS("' . date("Y-m-d", $args['search_from_date_added']) . '")', false);
    	}
    	if (isset($args['search_to_date_added'])) {
    		$this->db->where('TO_DAYS(r.date_added) <= ', 'TO_DAYS("' . date("Y-m-d", $args['search_to_date_added']) . '")', false);
    	}
    	$query = $this->db->get();
    	$row = $query->row_array('count_rows');
    	$this->paginator->setCount($row['count_rows']);

    	// -----------------------------------------------------------------------------------
    	// Select id of users that will be shown
    	//
    	$this->db->select('r.id');
    	$this->db->from('reviews AS r');
    	$this->db->where('r.objects_table', $objects_table);
    	if ($orderby)
    		$this->db->order_by($orderby, $direction);
    	if (isset($args['search_login'])) {
    		$this->db->join('users AS u', 'u.id=r.user_id', 'left');
    		$this->db->like('u.login', urldecode(html_entity_decode($args['search_login'])));
    	}
    	if (isset($args['search_anonyms'])) {
    		if ($args['search_anonyms'] == 'anonyms') {
    			$this->db->where('r.user_id', 0);
    		}
    	}
    	if (isset($args['search_status'])) {
    		$this->db->where('r.status', urldecode($args['search_status']));
    	}
    	if (isset($args['search_date_added'])) {
    		$this->db->where('TO_DAYS(r.date_added) = ', 'TO_DAYS("' . date("Y-m-d", $args['search_date_added']) . '")', false);
    	}
    	if (isset($args['search_from_date_added'])) {
    		$this->db->where('TO_DAYS(r.date_added) >= ', 'TO_DAYS("' . date("Y-m-d", $args['search_from_date_added']) . '")', false);
    	}
    	if (isset($args['search_to_date_added'])) {
    		$this->db->where('TO_DAYS(r.date_added) <= ', 'TO_DAYS("' . date("Y-m-d", $args['search_to_date_added']) . '")', false);
    	}
    	$query = $this->db->get();
    	$ids = $this->paginator->getResultIds($query->result_array());

    	if (!empty($ids)) {
	    	$this->db->select('*');
	    	$this->db->from('reviews AS r');
	    	$this->db->where('r.objects_table', $objects_table);
	    	$_array = array();
	    	foreach ($ids AS $id) {
	    		$_array[] = $id['id'];
	    	}
	    	$this->db->where_in('r.id', $_array);
	    	if ($orderby)
	    		$this->db->order_by($orderby, $direction);
	    	$query = $this->db->get();
	    	$reviews = array();
    		foreach ($query->result_array() AS $row) {
    			$review = new review($row);
    			$reviews[] = $review;
    		}
    		return $reviews;
    	} else {
    		return array();
    	}
    }
}
?>